@@ -32,7 +32,8 @@ G_BEGIN_DECLS
 #define CONF_UI_STATUSBAR_HIDDEN  CONF_PREFIX "/ui/statusbar_hidden"
 #define CONF_UI_SOURCELIST_HIDDEN CONF_PREFIX "/ui/sourcelist_hidden"
 #define CONF_UI_SMALL_DISPLAY     CONF_PREFIX "/ui/small_display"
-#define CONF_UI_COLUMNS_SETUP CONF_PREFIX "/ui/columns_setup"
+#define CONF_UI_OBSOLETE_COLUMNS_SETUP CONF_PREFIX "/ui/columns_setup"
+#define CONF_UI_COLUMNS_SETUP CONF_PREFIX "/ui/rhythmdb_columns_setup"
 #define CONF_UI_BROWSER_VIEWS CONF_PREFIX "/ui/browser_views"
 #define CONF_STATE_SHUFFLE	CONF_PREFIX "/state/shuffle"
 #define CONF_STATE_REPEAT	CONF_PREFIX "/state/repeat"
