@@ -35,17 +35,22 @@
 #include <gtk/gtkliststore.h>
 #include <libxml/entities.h>
 #include <libxml/SAX.h>
+#include <libxml/parserInternals.h>
 
 #include "rhythmdb-tree.h"
+#include "rhythmdb-query-model.h"
+#include "rhythmdb-property-model.h"
 #include "rb-debug.h"
 #include "rb-util.h"
+#include "rb-file-helpers.h"
+#include "rb-atomic.h"
 #include "rb-string-helpers.h"
 
 static void rhythmdb_tree_class_init (RhythmDBTreeClass *klass);
 static void rhythmdb_tree_init (RhythmDBTree *shell_player);
 static void rhythmdb_tree_finalize (GObject *object);
 
-static void rhythmdb_tree_load (RhythmDB *rdb, GMutex *mutex, gboolean *die);
+static void rhythmdb_tree_load (RhythmDB *rdb, gboolean *die);
 static void rhythmdb_tree_save (RhythmDB *rdb);
 static RhythmDBEntry * rhythmdb_tree_entry_new (RhythmDB *db, RhythmDBEntryType type,
 						const char *uri);
@@ -55,43 +60,17 @@ static void rhythmdb_tree_entry_set (RhythmDB *db, RhythmDBEntry *entry,
 static void rhythmdb_tree_entry_get (RhythmDB *db, RhythmDBEntry *entry,
 				guint propid, GValue *value);
 static void rhythmdb_tree_entry_delete (RhythmDB *db, RhythmDBEntry *entry);
-/* static GtkTreeModel * rhythmdb_tree_do_entry_query (RhythmDB *db, RhythmDBQuery *query); */
-/* static GtkTreeModel * rhythmdb_tree_do_property_query (RhythmDB *db, guint property_id, RhythmDBQuery *query); */
 static RhythmDBEntry * rhythmdb_tree_entry_lookup_by_location (RhythmDB *db, const char *uri);
 static void rhythmdb_tree_do_full_query (RhythmDB *db, GPtrArray *query,
-					 GtkTreeModel **main_model,
-					 GtkTreeModel **genre_model,
-					 GtkTreeModel **artist_model,
-					 GtkTreeModel **album_model);
+					 GtkTreeModel *main_model, gboolean *cancel);
+gboolean rhythmdb_tree_evaluate_query (RhythmDB *adb, GPtrArray *query,
+				       RhythmDBEntry *aentry);
 
 #define RHYTHMDB_TREE_XML_VERSION "1.0"
 
-typedef struct RhythmDBTreeProperty
-{
-#ifndef G_DISABLE_ASSERT
-	guint magic;
-#endif	
-	struct RhythmDBTreeProperty *parent;
-	char *name;
-	char *sort_key;
-	GHashTable *children;
-} RhythmDBTreeProperty;
-
 #define RHYTHMDB_TREE_ENTRY_VALUE(ENTRY, PROPID) (&((ENTRY)->properties[PROPID]))
 #define RHYTHMDB_TREE_ENTRY_GET_TYPE(ENTRY) (g_value_get_int (RHYTHMDB_TREE_ENTRY_VALUE (ENTRY, RHYTHMDB_PROP_TYPE)))
 
-/* Optimization possibility - note that we aren't using at least
- * three values in the array; the genre/artist/album names are
- * actually stored in the tree structure. */
-typedef struct
-{
-#ifndef G_DISABLE_ASSERT
-	guint magic;
-#endif	
-	RhythmDBTreeProperty *album;
-	GValue properties[RHYTHMDB_NUM_PROPERTIES];
-} RhythmDBTreeEntry;
-
 #ifndef G_DISABLE_ASSERT
 static inline RhythmDBTreeProperty *
 assert_valid_tree_property (RhythmDBTreeProperty *prop)
@@ -116,7 +95,7 @@ assert_valid_tree_entry (RhythmDBTreeEntry *entry)
 #define RHYTHMDB_TREE_ENTRY(x) ((RhythmDBEntry *) x)
 #endif
 
-static RhythmDBEntry *rhythmdb_tree_entry_allocate (RhythmDBTree *db);
+static RhythmDBEntry *rhythmdb_tree_entry_allocate (RhythmDBTree *db, RhythmDBEntryType type);
 static void rhythmdb_tree_entry_insert (RhythmDBTree *db, RhythmDBTreeEntry *entry,
 					RhythmDBEntryType type, const char *uri,
 					const char *genrename, const char *artistname,
@@ -139,11 +118,18 @@ static char *get_entry_album_name (RhythmDBTreeEntry *entry);
 static char *get_entry_genre_sort_key (RhythmDBTreeEntry *entry);
 static char *get_entry_artist_sort_key (RhythmDBTreeEntry *entry);
 static char *get_entry_album_sort_key (RhythmDBTreeEntry *entry);
+static char *get_entry_genre_folded (RhythmDBTreeEntry *entry);
+static char *get_entry_artist_folded (RhythmDBTreeEntry *entry);
+static char *get_entry_album_folded (RhythmDBTreeEntry *entry);
 
 static void handle_genre_deletion (RhythmDBTree *db, const char *name);
 static void handle_artist_deletion (RhythmDBTree *db, const char *name);
 static void handle_album_deletion (RhythmDBTree *db, const char *name);
 
+static GList *split_query_by_disjunctions (RhythmDBTree *db, GPtrArray *query);
+static gboolean evaluate_conjunctive_subquery (RhythmDBTree *db, GPtrArray *query,
+					       guint base, guint max, RhythmDBTreeEntry *entry);
+
 struct RhythmDBTreePrivate
 {
 	GMemChunk *entry_memchunk;
@@ -211,6 +197,7 @@ rhythmdb_tree_class_init (RhythmDBTreeClass *klass)
 	rhythmdb_class->impl_entry_get = rhythmdb_tree_entry_get;
 	rhythmdb_class->impl_entry_delete = rhythmdb_tree_entry_delete;
 	rhythmdb_class->impl_lookup_by_location = rhythmdb_tree_entry_lookup_by_location;
+	rhythmdb_class->impl_evaluate_query = rhythmdb_tree_evaluate_query;
 	rhythmdb_class->impl_do_full_query = rhythmdb_tree_do_full_query;
 }
 
@@ -220,8 +207,8 @@ elt_name_from_propid (RhythmDBPropType propid)
 	switch (propid) {
 	case RHYTHMDB_PROP_TYPE:
 		return "type";
-	case RHYTHMDB_PROP_NAME:
-		return "name";
+	case RHYTHMDB_PROP_TITLE:
+		return "title";
 	case RHYTHMDB_PROP_GENRE:
 		return "genre";
 	case RHYTHMDB_PROP_ARTIST:
@@ -257,9 +244,7 @@ rhythmdb_tree_init (RhythmDBTree *db)
 	guint i;
 	db->priv = g_new0 (RhythmDBTreePrivate, 1);
 
-	db->priv->entries = g_hash_table_new_full (g_str_hash, g_str_equal,
-						   NULL,
-						   (GDestroyNotify) rhythmdb_tree_entry_finalize);
+	db->priv->entries = g_hash_table_new (g_str_hash, g_str_equal);
 
 	db->priv->entry_memchunk = g_mem_chunk_new ("RhythmDBTree entry memchunk",
 						    sizeof (RhythmDBTreeEntry),
@@ -330,34 +315,54 @@ get_entry_album_sort_key (RhythmDBTreeEntry *entry)
 	return entry->album->sort_key;
 }
 
-#ifndef G_DISABLE_ASSERT
+static inline char *
+get_entry_genre_folded (RhythmDBTreeEntry *entry)
+{
+	return entry->album->parent->parent->folded;
+}
+
+static inline char *
+get_entry_artist_folded (RhythmDBTreeEntry *entry)
+{
+	return entry->album->parent->folded;
+}
+
+static inline char *
+get_entry_album_folded (RhythmDBTreeEntry *entry)
+{
+	return entry->album->folded;
+}
+
 static inline void
 sanity_check_entry_tree (RhythmDBTreeEntry *entry)
 {
+#ifdef RHYTHMDB_ENABLE_SANITY_CHECK
 	RHYTHMDB_TREE_ENTRY (entry); 
 	RHYTHMDB_TREE_PROPERTY (entry->album); 
 	RHYTHMDB_TREE_PROPERTY (entry->album->parent); 
 	RHYTHMDB_TREE_PROPERTY (entry->album->parent->parent); 
+#endif
 }
+#ifdef RHYTHMDB_ENABLE_SANITY_CHECK
 static void
 sanity_check_entry_tree_from_hash (gpointer unused, RhythmDBTreeEntry *entry)
 {
 	sanity_check_entry_tree (entry);
 }
+#endif
 static void
 sanity_check_database (RhythmDBTree *db)
 {
+#ifdef RHYTHMDB_ENABLE_SANITY_CHECK
 	g_hash_table_foreach (db->priv->entries, (GHFunc) sanity_check_entry_tree_from_hash, NULL);
-}
-#else
-#define sanity_check_entry_tree(entry)
-#define sanity_check_database(db)
 #endif
+}
 
 static void
 unparent_entries (const char *uri, RhythmDBTreeEntry *entry, RhythmDBTree *db)
 {
 	remove_entry_from_album (db, entry);
+	rhythmdb_tree_entry_finalize (entry);
 }
 
 static void
@@ -396,7 +401,7 @@ rhythmdb_tree_finalize (GObject *object)
 struct RhythmDBTreeLoadContext
 {
 	RhythmDBTree *db;
-	GMutex *mutex;
+	xmlParserCtxtPtr xmlctx;
 	gboolean *die;
 	enum {
 		RHYTHMDB_TREE_PARSER_STATE_START,
@@ -417,10 +422,10 @@ static void
 rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 				    const char *name, const char **attrs)
 {
-	g_mutex_lock (ctx->mutex);
-	if (*ctx->die == TRUE)
-		ctx->state = RHYTHMDB_TREE_PARSER_STATE_END;
-	g_mutex_unlock (ctx->mutex);
+	if (*ctx->die == TRUE) {
+		xmlStopParser (ctx->xmlctx);
+		return;
+	}
 
 	switch (ctx->state)
 	{
@@ -433,8 +438,22 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 	case RHYTHMDB_TREE_PARSER_STATE_RHYTHMDB:
 	{
 		if (!strcmp (name, "entry")) {
+			RhythmDBEntryType type;
+			gboolean type_set = FALSE;
+			for (; *attrs; attrs +=2) {
+				if (!strcmp (*attrs, "type")) {
+					const char *typename = *(attrs+1);
+					if (!strcmp (typename, "song"))
+						type = RHYTHMDB_ENTRY_TYPE_SONG;
+					else 
+						type = RHYTHMDB_ENTRY_TYPE_IRADIO_STATION;
+					type_set = TRUE;
+					break;
+				}
+			}
+			g_assert (type_set);
 			ctx->state = RHYTHMDB_TREE_PARSER_STATE_ENTRY;
-			ctx->entry = rhythmdb_tree_entry_allocate (ctx->db);
+			ctx->entry = rhythmdb_tree_entry_allocate (ctx->db, type);
 			ctx->genrename = NULL;
 			ctx->albumname = NULL;
 			ctx->artistname = NULL;
@@ -461,10 +480,10 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 static void
 rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const char *name)
 {
-	g_mutex_lock (ctx->mutex);
-	if (*ctx->die == TRUE)
-		ctx->state = RHYTHMDB_TREE_PARSER_STATE_END;
-	g_mutex_unlock (ctx->mutex);
+	if (*ctx->die == TRUE) {
+		xmlStopParser (ctx->xmlctx);
+		return;
+	}
 	
 	switch (ctx->state)
 	{
@@ -486,6 +505,11 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const cha
 		rhythmdb_tree_entry_insert (ctx->db, ctx->entry, type,
 					    uri, ctx->genrename, ctx->artistname,
 					    ctx->albumname);
+		g_free (ctx->genrename);
+		g_free (ctx->artistname);
+		g_free (ctx->albumname);
+
+		rhythmdb_emit_entry_restored (RHYTHMDB (ctx->db), ctx->entry);
 
 		rhythmdb_write_unlock (RHYTHMDB (ctx->db));
 
@@ -550,6 +574,8 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const cha
 			g_assert_not_reached ();
 			break;
 		}
+		rhythmdb_entry_sync_mirrored (RHYTHMDB (ctx->db),
+					      ctx->entry, ctx->propid, value);
 
 	nextstate:
 		ctx->state = RHYTHMDB_TREE_PARSER_STATE_ENTRY;
@@ -565,10 +591,10 @@ static void
 rhythmdb_tree_parser_characters (struct RhythmDBTreeLoadContext *ctx, const char *data,
 				 guint len)
 {
-	g_mutex_lock (ctx->mutex);
-	if (*ctx->die == TRUE)
-		ctx->state = RHYTHMDB_TREE_PARSER_STATE_END;
-	g_mutex_unlock (ctx->mutex);
+	if (*ctx->die == TRUE) {
+		xmlStopParser (ctx->xmlctx);
+		return;
+	}
 
 	switch (ctx->state)
 	{
@@ -584,9 +610,10 @@ rhythmdb_tree_parser_characters (struct RhythmDBTreeLoadContext *ctx, const char
 }
 
 static void
-rhythmdb_tree_load (RhythmDB *rdb, GMutex *mutex, gboolean *die)
+rhythmdb_tree_load (RhythmDB *rdb, gboolean *die)
 {
 	RhythmDBTree *db = RHYTHMDB_TREE (rdb);
+	xmlParserCtxtPtr ctxt;
 	xmlSAXHandlerPtr sax_handler = g_new0 (xmlSAXHandler, 1);
 	struct RhythmDBTreeLoadContext *ctx = g_new0 (struct RhythmDBTreeLoadContext, 1);
 	char *name;
@@ -597,12 +624,21 @@ rhythmdb_tree_load (RhythmDB *rdb, GMutex *mutex, gboolean *die)
 
 	ctx->state = RHYTHMDB_TREE_PARSER_STATE_START;
 	ctx->db = db;
-	ctx->mutex = mutex;
 	ctx->die = die;
 
 	g_object_get (G_OBJECT (db), "name", &name, NULL);
 
-	xmlSAXUserParseFile (sax_handler, ctx, name);
+	if (rb_uri_exists (name)) {
+		ctxt = xmlCreateFileParserCtxt (name);
+		ctx->xmlctx = ctxt;
+		xmlFree (ctxt->sax);
+		ctxt->userData = ctx;
+		ctxt->sax = sax_handler;
+		xmlParseDocument (ctxt);
+		ctxt->sax = NULL;
+		xmlFreeParserCtxt (ctxt);
+			
+	}
 	g_free (name);
 	g_free (sax_handler);
 	g_free (ctx);
@@ -655,7 +691,7 @@ g_free (encoded);							\
 		RHYTHMDB_FWRITE_STATICSTR ("iradio", ctx->handle);
 		break;
 	}
-	RHYTHMDB_FWRITE_STATICSTR ("\">", ctx->handle);
+	RHYTHMDB_FWRITE_STATICSTR ("\">\n", ctx->handle);
 		
 	/* Skip over the first property - the type */
 	for (i = 1; i < RHYTHMDB_NUM_SAVED_PROPERTIES; i++) {
@@ -759,7 +795,6 @@ rhythmdb_tree_save (RhythmDB *rdb)
 
 	fprintf (f, "%s\n", "</rhythmdb>");
 
-	fsync (fileno (f));
 	fclose (f);
 
 	rename (savepath->str, name);
@@ -789,7 +824,7 @@ set_entry_album (RhythmDBTree *db, RhythmDBTreeEntry *entry, RhythmDBTreePropert
 }
 
 static RhythmDBEntry *
-rhythmdb_tree_entry_allocate (RhythmDBTree *db)
+rhythmdb_tree_entry_allocate (RhythmDBTree *db, RhythmDBEntryType type)
 {
 	RhythmDBTreeEntry *ret;
 	guint i;
@@ -799,6 +834,7 @@ rhythmdb_tree_entry_allocate (RhythmDBTree *db)
 #ifndef G_DISABLE_ASSERT
 	ret->magic = 0xdeadb33f;
 #endif	
+	ret->refcount.value = 1;
 
 	/* Initialize all the properties. */
 	for (i = 0; i < RHYTHMDB_NUM_PROPERTIES; i++) {
@@ -808,7 +844,12 @@ rhythmdb_tree_entry_allocate (RhythmDBTree *db)
 		/* Hack to ensure all string values are initialized. */
 		if (val_type == G_TYPE_STRING)
 			g_value_set_static_string (RHYTHMDB_TREE_ENTRY_VALUE (ret, i), "");
+
+		if (i == RHYTHMDB_PROP_TRACK_NUMBER)
+			g_value_set_int (RHYTHMDB_TREE_ENTRY_VALUE (ret, i), -1);
 	}
+	g_value_set_int (RHYTHMDB_TREE_ENTRY_VALUE (ret, RHYTHMDB_PROP_TYPE), type);
+
 	return ret;
 }
 
@@ -823,6 +864,9 @@ rhythmdb_tree_entry_insert (RhythmDBTree *db, RhythmDBTreeEntry *entry,
 	RhythmDBTreeProperty *genre;
 	char *new_uri;
 
+	if (g_hash_table_lookup (db->priv->entries, uri))
+		return;
+
 	/* Initialize the tree structure. */
 	genre = get_or_create_genre (db, type, genrename);
 	artist = get_or_create_artist (db, genre, artistname);
@@ -830,7 +874,6 @@ rhythmdb_tree_entry_insert (RhythmDBTree *db, RhythmDBTreeEntry *entry,
 
 	new_uri = g_strdup (uri);
 
-	g_assert (!g_hash_table_lookup (db->priv->entries, new_uri));
 	g_hash_table_insert (db->priv->entries, new_uri, entry);
 	g_value_set_string_take_ownership (RHYTHMDB_TREE_ENTRY_VALUE (entry, RHYTHMDB_PROP_LOCATION),
 					   new_uri);
@@ -845,23 +888,30 @@ rhythmdb_tree_entry_new (RhythmDB *rdb, RhythmDBEntryType type, const char *uri)
 
 	sanity_check_database (db);
 
-	ret = rhythmdb_tree_entry_allocate (db);
+	ret = rhythmdb_tree_entry_allocate (db, type);
 
 	rhythmdb_tree_entry_insert (db, ret, type, uri, "", "", "");
 
-	g_value_set_int (RHYTHMDB_TREE_ENTRY_VALUE (ret, RHYTHMDB_PROP_TYPE), type);
-
 	sanity_check_database (db);
 
 	return ret;
 }
 
+void
+rhythmdb_tree_entry_destroy (RhythmDBTree *db, RhythmDBEntry *aentry)
+{
+	RhythmDBTreeEntry *entry = RHYTHMDB_TREE_ENTRY (aentry);
+	rhythmdb_tree_entry_finalize (entry);
+	g_mem_chunk_free (db->priv->entry_memchunk, entry);
+}
+
 static RhythmDBTreeProperty *
 rhythmdb_tree_property_new (RhythmDBTree *db, const char *name)
 {
 	RhythmDBTreeProperty *ret = g_mem_chunk_alloc0 (db->priv->property_memchunk);
 	ret->name = g_strdup (name);
 	ret->sort_key = rb_get_sort_key (ret->name);
+	ret->folded = g_utf8_casefold (ret->name, -1);
 #ifndef G_DISABLE_ASSERT
 	ret->magic = 0xf00dbeef;
 #endif	
@@ -894,7 +944,7 @@ get_or_create_genre (RhythmDBTree *db, RhythmDBEntryType type,
 							 NULL);
 		g_hash_table_insert (table, g_strdup (name), genre);
 		genre->parent = NULL;
-		rhythmdb_emit_genre_added (RHYTHMDB (db), name);
+		/* rhythmdb_emit_genre_added (RHYTHMDB (db), name); */
 	}
 
 	return RHYTHMDB_TREE_PROPERTY (genre);
@@ -915,7 +965,7 @@ get_or_create_artist (RhythmDBTree *db, RhythmDBTreeProperty *genre,
 							  NULL);
 		g_hash_table_insert (genre->children, g_strdup (name), artist);
 		artist->parent = genre;
-		rhythmdb_emit_artist_added (RHYTHMDB (db), name);
+		/* rhythmdb_emit_artist_added (RHYTHMDB (db), name); */
 	}
 
 	return RHYTHMDB_TREE_PROPERTY (artist);
@@ -934,31 +984,31 @@ get_or_create_album (RhythmDBTree *db, RhythmDBTreeProperty *artist,
 		album->children = g_hash_table_new (g_direct_hash, g_direct_equal);
 		g_hash_table_insert (artist->children, g_strdup (name), album);
 		album->parent = artist;
-		rhythmdb_emit_album_added (RHYTHMDB (db), name);
+		/* rhythmdb_emit_album_added (RHYTHMDB (db), name); */
 	}
 
 	return RHYTHMDB_TREE_PROPERTY (album);
 }
 
-static void
+static inline void
 handle_genre_deletion (RhythmDBTree *db, const char *name)
 {
-	if (!db->priv->finalizing)
-		rhythmdb_emit_genre_deleted (RHYTHMDB (db), name);
+/* 	if (!db->priv->finalizing) */
+/* 		rhythmdb_emit_genre_deleted (RHYTHMDB (db), name); */
 }
 
-static void
+static inline void
 handle_artist_deletion (RhythmDBTree *db, const char *name)
 {
-	if (!db->priv->finalizing)
-		rhythmdb_emit_artist_deleted (RHYTHMDB (db), name);
+/* 	if (!db->priv->finalizing) */
+/* 		rhythmdb_emit_artist_deleted (RHYTHMDB (db), name); */
 }
 
-static void
+static inline void
 handle_album_deletion (RhythmDBTree *db, const char *name)
 {
-	if (!db->priv->finalizing)
-		rhythmdb_emit_album_deleted (RHYTHMDB (db), name);
+/* 	if (!db->priv->finalizing) */
+/* 		rhythmdb_emit_album_deleted (RHYTHMDB (db), name); */
 }
 
 static gboolean
@@ -1030,6 +1080,9 @@ rhythmdb_tree_entry_set (RhythmDB *adb, RhythmDBEntry *aentry,
 
 	sanity_check_database (db);
 
+	if (entry->deleted)
+		return;
+
 	/* Handle special properties */
 	switch (propid)
 	{
@@ -1128,33 +1181,61 @@ rhythmdb_tree_entry_set (RhythmDB *adb, RhythmDBEntry *aentry,
 }
 
 static void
-rhythmdb_tree_entry_get (RhythmDB *db, RhythmDBEntry *aentry,
+rhythmdb_tree_entry_get (RhythmDB *adb, RhythmDBEntry *aentry,
 			 guint propid, GValue *value)
 {
-	RhythmDBTreeEntry *entry = aentry;
+	RhythmDBTree *db = RHYTHMDB_TREE (adb);
+	RhythmDBTreeEntry *entry = RHYTHMDB_TREE_ENTRY (aentry);
 
-	sanity_check_database (RHYTHMDB_TREE (db));
+	sanity_check_database (db);
+
+	if (G_UNLIKELY (entry->deleted)) {
+		g_value_copy (RHYTHMDB_TREE_ENTRY_VALUE (entry, propid), value);
+		return;
+	}
 
-	/* Handle special properties */
 	switch (propid)
 	{
+	/* Handle special properties */
 	case RHYTHMDB_PROP_ALBUM:
-		g_value_set_string (value, get_entry_album_name (entry));
+		g_value_set_static_string (value, get_entry_album_name (entry));
 		break;
 	case RHYTHMDB_PROP_ARTIST:
-		g_value_set_string (value, get_entry_artist_name (entry));
+		g_value_set_static_string (value, get_entry_artist_name (entry));
 		break;
 	case RHYTHMDB_PROP_GENRE:
-		g_value_set_string (value, get_entry_genre_name (entry));
+		g_value_set_static_string (value, get_entry_genre_name (entry));
 		break;
 	case RHYTHMDB_PROP_ALBUM_SORT_KEY:
-		g_value_set_string (value, get_entry_album_sort_key (entry));
+		g_value_set_static_string (value, get_entry_album_sort_key (entry));
 		break;
 	case RHYTHMDB_PROP_ARTIST_SORT_KEY:
-		g_value_set_string (value, get_entry_artist_sort_key (entry));
+		g_value_set_static_string (value, get_entry_artist_sort_key (entry));
 		break;
 	case RHYTHMDB_PROP_GENRE_SORT_KEY:
-		g_value_set_string (value, get_entry_genre_sort_key (entry));
+		g_value_set_static_string (value, get_entry_genre_sort_key (entry));
+		break;
+	case RHYTHMDB_PROP_ALBUM_FOLDED:
+		g_value_set_static_string (value, get_entry_album_folded (entry));
+		break;
+	case RHYTHMDB_PROP_ARTIST_FOLDED:
+		g_value_set_static_string (value, get_entry_artist_folded (entry));
+		break;
+	case RHYTHMDB_PROP_GENRE_FOLDED:
+		g_value_set_static_string (value, get_entry_genre_folded (entry));
+		break;
+	/* Handle other string properties */
+	case RHYTHMDB_PROP_TITLE:
+		g_value_set_static_string (value, g_value_get_string (RHYTHMDB_TREE_ENTRY_VALUE (entry, RHYTHMDB_PROP_TITLE)));
+		break;
+	case RHYTHMDB_PROP_TITLE_SORT_KEY:
+		g_value_set_static_string (value, g_value_get_string (RHYTHMDB_TREE_ENTRY_VALUE (entry, RHYTHMDB_PROP_TITLE_SORT_KEY)));
+		break;
+	case RHYTHMDB_PROP_LOCATION:
+		g_value_set_static_string (value, g_value_get_string (RHYTHMDB_TREE_ENTRY_VALUE (entry, RHYTHMDB_PROP_LOCATION)));
+		break;
+	case RHYTHMDB_PROP_LAST_PLAYED_STR:
+		g_value_set_static_string (value, g_value_get_string (RHYTHMDB_TREE_ENTRY_VALUE (entry, RHYTHMDB_PROP_LAST_PLAYED_STR)));
 		break;
 	default:
 		g_value_copy (RHYTHMDB_TREE_ENTRY_VALUE (entry, propid), value);
@@ -1187,15 +1268,29 @@ rhythmdb_tree_entry_delete (RhythmDB *adb, RhythmDBEntry *aentry)
 
 	sanity_check_database (db);
 
+	if (entry->deleted)
+		return;
+
+	entry->deleted = TRUE;
+
 #ifndef G_DISABLE_ASSERT
 	uri = g_value_get_string (RHYTHMDB_TREE_ENTRY_VALUE (entry, RHYTHMDB_PROP_LOCATION));
 	g_assert (g_hash_table_lookup (db->priv->entries, uri) != NULL);
 #endif
-
+	
+	/* We store these properties back in the entry temporarily so that later
+	   callbacks can retreive the value even though the entry is removed from
+	   the indexed tree.
+	*/
+	g_value_set_string (RHYTHMDB_TREE_ENTRY_VALUE (entry, RHYTHMDB_PROP_GENRE),
+			    get_entry_genre_name (entry));
+	g_value_set_string (RHYTHMDB_TREE_ENTRY_VALUE (entry, RHYTHMDB_PROP_ARTIST),
+			    get_entry_artist_name (entry));
+	g_value_set_string (RHYTHMDB_TREE_ENTRY_VALUE (entry, RHYTHMDB_PROP_ALBUM),
+			    get_entry_album_name (entry));
 	remove_entry_from_album (db, entry); 
-
+	
 	g_hash_table_remove (db->priv->entries, uri);
-	g_mem_chunk_free (db->priv->entry_memchunk, entry);
 
 	sanity_check_database (db);
 }
@@ -1208,10 +1303,12 @@ destroy_tree_property (RhythmDBTreeProperty *prop)
 #endif
 	g_free (prop->name);
 	g_free (prop->sort_key);
+	g_free (prop->folded);
 	g_hash_table_destroy (prop->children);
 }
 
 typedef void (*RhythmDBTreeTraversalFunc) (RhythmDBTree *db, RhythmDBTreeEntry *entry, gpointer data);
+typedef void (*RhythmDBTreeAlbumTraversalFunc) (RhythmDBTree *db, RhythmDBTreeProperty *album, gpointer data);
 
 struct RhythmDBTreeTraversalData
 {
@@ -1219,43 +1316,98 @@ struct RhythmDBTreeTraversalData
 	GPtrArray *query;
 	RhythmDBTreeTraversalFunc func;
 	gpointer data;
+	gboolean *cancel;
 };
 
+gboolean
+rhythmdb_tree_evaluate_query (RhythmDB *adb, GPtrArray *query,
+			      RhythmDBEntry *aentry)
+{
+	RhythmDBTree *db = RHYTHMDB_TREE (adb);
+	RhythmDBTreeEntry *entry = RHYTHMDB_TREE_ENTRY (aentry);
+	guint i;
+	guint last_disjunction;
+
+	for (i = 0, last_disjunction = 0; i < query->len; i++) {
+		RhythmDBQueryData *data = g_ptr_array_index (query, i);
+
+		if (data->type == RHYTHMDB_QUERY_DISJUNCTION) {
+			if (evaluate_conjunctive_subquery (db, query, last_disjunction, i, entry))
+				return TRUE;
+
+			last_disjunction = i;
+		}
+	}
+	if (evaluate_conjunctive_subquery (db, query, last_disjunction, query->len, entry))
+		return TRUE;
+	return FALSE;
+}
+
 static gboolean
-evaluate_conjunctive_subquery (RhythmDBTree *db, GPtrArray *query, RhythmDBTreeEntry *entry)
+evaluate_conjunctive_subquery (RhythmDBTree *db, GPtrArray *query,
+			       guint base, guint max, RhythmDBTreeEntry *entry)
+
 {
 	guint i;
+	entry = RHYTHMDB_TREE_ENTRY (entry);
 /* Optimization possibility - we may get here without actually having
  * anything in the query.  It would be faster to instead just merge
  * the child hash table into the query result hash.
  */
-	for (i = 0; i < query->len; i++) {
+	for (i = base; i < max; i++) {
 		RhythmDBQueryData *data = g_ptr_array_index (query, i);
 
 		switch (data->type) {
+		case RHYTHMDB_QUERY_SUBQUERY:
+		{
+			gboolean matched = FALSE;
+			GList *conjunctions = split_query_by_disjunctions (db, data->subquery);
+			GList *tem;
+
+			for (tem = conjunctions; tem; tem = tem->next) {
+				GPtrArray *subquery = tem->data;
+				if (!matched && evaluate_conjunctive_subquery (db, subquery,
+									       0, subquery->len,
+									       entry)) {
+					matched = TRUE;
+				}
+				g_ptr_array_free (tem->data, TRUE);
+			}
+			g_list_free (conjunctions);
+			if (!matched)
+				return FALSE;
+		}
+		break;
 		case RHYTHMDB_QUERY_PROP_LIKE:
 			if (G_VALUE_TYPE (data->val) == G_TYPE_STRING) {
+				gboolean islike;
 				const char *stra, *strb;
 
 				switch (data->propid)
 				{
-				case RHYTHMDB_PROP_ALBUM:
-					stra = get_entry_album_name (entry);
+				case RHYTHMDB_PROP_ALBUM_FOLDED:
+					stra = get_entry_album_folded (entry);
 					break;
-				case RHYTHMDB_PROP_ARTIST:
-					stra = get_entry_artist_name (entry);
+				case RHYTHMDB_PROP_ARTIST_FOLDED:
+					stra = get_entry_artist_folded (entry);
 					break;
-				case RHYTHMDB_PROP_GENRE:
-					stra = get_entry_genre_name (entry);
+				case RHYTHMDB_PROP_GENRE_FOLDED:
+					stra = get_entry_genre_folded (entry);
 					break;
 				default:
 					stra = g_value_get_string (RHYTHMDB_TREE_ENTRY_VALUE (entry, data->propid));
 				}
-		
+
 				strb = g_value_get_string (data->val);
-				return strstr (strb, stra) != NULL;
+				islike = (strstr (stra, strb) != NULL);
+				if (!islike)
+					return FALSE;
+			} else {
+				if (rb_gvalue_compare (RHYTHMDB_TREE_ENTRY_VALUE (entry, data->propid),
+						       data->val) != 0)
+					return FALSE;
 			}
-			/* Deliberately fall through here */
+			break;
 		case RHYTHMDB_QUERY_PROP_EQUALS:
 			switch (data->propid)
 			{
@@ -1304,18 +1456,36 @@ static void
 do_conjunction (RhythmDBTreeEntry *entry, gpointer unused,
 		struct RhythmDBTreeTraversalData *data)
 {
+	if (G_UNLIKELY (*data->cancel))
+		return;
 	/* Finally, we actually evaluate the query! */
-	if (evaluate_conjunctive_subquery (data->db, data->query, entry))
+	if (evaluate_conjunctive_subquery (data->db, data->query, 0, data->query->len,
+					   entry)) {
 		data->func (data->db, entry, data->data);
+	}
 }
 
 static void
 conjunctive_query_songs (const char *name, RhythmDBTreeProperty *album,
 			 struct RhythmDBTreeTraversalData *data)
 {
+	if (G_UNLIKELY (*data->cancel))
+		return;
 	g_hash_table_foreach (album->children, (GHFunc) do_conjunction, data);
 }
 
+static GPtrArray *
+clone_remove_ptr_array_index (GPtrArray *arr, guint index)
+{
+	GPtrArray *ret = g_ptr_array_new ();
+	guint i;
+	for (i = 0; i < arr->len; i++)
+		if (i != index)
+			g_ptr_array_add (ret, g_ptr_array_index (arr, i));
+	
+	return ret;
+}
+
 static void
 conjunctive_query_albums (const char *name, RhythmDBTreeProperty *artist,
 			  struct RhythmDBTreeTraversalData *data)
@@ -1323,6 +1493,9 @@ conjunctive_query_albums (const char *name, RhythmDBTreeProperty *artist,
 	guint i;
 	int album_query_idx = -1;
 
+	if (G_UNLIKELY (*data->cancel))
+		return;
+
 	for (i = 0; i < data->query->len; i++) {
 		RhythmDBQueryData *qdata = g_ptr_array_index (data->query, i);
 		if (qdata->type == RHYTHMDB_QUERY_PROP_EQUALS
@@ -1337,14 +1510,17 @@ conjunctive_query_albums (const char *name, RhythmDBTreeProperty *artist,
 	if (album_query_idx >= 0) {
 		RhythmDBTreeProperty *album;
 		RhythmDBQueryData *qdata = g_ptr_array_index (data->query, album_query_idx);
-		
-		g_ptr_array_remove_index_fast (data->query, album_query_idx);
+		GPtrArray *oldquery = data->query;
+
+		data->query = clone_remove_ptr_array_index (data->query, album_query_idx);
 		
 		album = g_hash_table_lookup (artist->children, g_value_get_string (qdata->val));
 
 		if (album != NULL) {
-			conjunctive_query_songs (album->name, album, data);
+				conjunctive_query_songs (album->name, album, data);
 		}
+		g_ptr_array_free (data->query, TRUE);
+		data->query = oldquery;
 		return;
 	} 
 
@@ -1358,6 +1534,9 @@ conjunctive_query_artists (const char *name, RhythmDBTreeProperty *genre,
 	guint i;
 	int artist_query_idx = -1;
 
+	if (G_UNLIKELY (*data->cancel))
+		return;
+
 	for (i = 0; i < data->query->len; i++) {
 		RhythmDBQueryData *qdata = g_ptr_array_index (data->query, i);
 		if (qdata->type == RHYTHMDB_QUERY_PROP_EQUALS
@@ -1372,13 +1551,16 @@ conjunctive_query_artists (const char *name, RhythmDBTreeProperty *genre,
 	if (artist_query_idx >= 0) {
 		RhythmDBTreeProperty *artist;
 		RhythmDBQueryData *qdata = g_ptr_array_index (data->query, artist_query_idx);
-		
-		g_ptr_array_remove_index_fast (data->query, artist_query_idx);
+		GPtrArray *oldquery = data->query;
+
+		data->query = clone_remove_ptr_array_index (data->query, artist_query_idx);
 		
 		artist = g_hash_table_lookup (genre->children, g_value_get_string (qdata->val));
 		if (artist != NULL) {
 			conjunctive_query_albums (artist->name, artist, data);
 		}
+		g_ptr_array_free (data->query, TRUE);
+		data->query = oldquery;
 		return;
 	} 
 
@@ -1387,13 +1569,16 @@ conjunctive_query_artists (const char *name, RhythmDBTreeProperty *genre,
 
 static void
 conjunctive_query_genre (RhythmDBTree *db, GHashTable *genres,
-			 GPtrArray *query, struct RhythmDBTreeTraversalData *data)
+			 struct RhythmDBTreeTraversalData *data)
 {
 	int genre_query_idx = -1;
 	guint i;
+
+	if (G_UNLIKELY (*data->cancel))
+		return;
 	
-	for (i = 0; i < query->len; i++) {
-		RhythmDBQueryData *qdata = g_ptr_array_index (query, i);
+	for (i = 0; i < data->query->len; i++) {
+		RhythmDBQueryData *qdata = g_ptr_array_index (data->query, i);
 		if (qdata->type == RHYTHMDB_QUERY_PROP_EQUALS
 		    && qdata->propid == RHYTHMDB_PROP_GENRE) {
 			/* A song can't currently have two genres.  So
@@ -1408,14 +1593,17 @@ conjunctive_query_genre (RhythmDBTree *db, GHashTable *genres,
 
 	if (genre_query_idx >= 0) {
 		RhythmDBTreeProperty *genre;
-		RhythmDBQueryData *qdata = g_ptr_array_index (query, genre_query_idx);
-		
-		g_ptr_array_remove_index_fast (query, genre_query_idx);
+		RhythmDBQueryData *qdata = g_ptr_array_index (data->query, genre_query_idx);
+		GPtrArray *oldquery = data->query;
+
+		data->query = clone_remove_ptr_array_index (data->query, genre_query_idx);
 		
 		genre = g_hash_table_lookup (genres, g_value_get_string (qdata->val));
 		if (genre != NULL) {
 			conjunctive_query_artists (genre->name, genre, data);
 		} 
+		g_ptr_array_free (data->query, TRUE);
+		data->query = oldquery;
 		return;
 	} 
 
@@ -1424,7 +1612,8 @@ conjunctive_query_genre (RhythmDBTree *db, GHashTable *genres,
 
 static void
 conjunctive_query (RhythmDBTree *db, GPtrArray *query,
-		   RhythmDBTreeTraversalFunc func, gpointer data)
+		   RhythmDBTreeTraversalFunc func, gpointer data,
+		   gboolean *cancel)
 {
 	int type_query_idx = -1;
 	guint i;
@@ -1446,6 +1635,7 @@ conjunctive_query (RhythmDBTree *db, GPtrArray *query,
 	traversal_data->query = query;
 	traversal_data->func = func;
 	traversal_data->data = data;
+	traversal_data->cancel = cancel;
 
 	if (type_query_idx >= 0) {
 		RhythmDBEntryType etype;
@@ -1457,18 +1647,18 @@ conjunctive_query (RhythmDBTree *db, GPtrArray *query,
 		switch (etype)
 		{
 		case RHYTHMDB_ENTRY_TYPE_SONG:
-			conjunctive_query_genre (db, db->priv->song_genres, query, traversal_data);
+			conjunctive_query_genre (db, db->priv->song_genres, traversal_data);
 			break;
 		case RHYTHMDB_ENTRY_TYPE_IRADIO_STATION:
-			conjunctive_query_genre (db, db->priv->iradio_genres, query, traversal_data);
+			conjunctive_query_genre (db, db->priv->iradio_genres, traversal_data);
 			break;
 		}
 		goto out;
 	} 
 
 	/* No type was given; punt and query everything */
-	conjunctive_query_genre (db, db->priv->song_genres, query, traversal_data);
-	conjunctive_query_genre (db, db->priv->iradio_genres, query, traversal_data);
+	conjunctive_query_genre (db, db->priv->song_genres, traversal_data);
+	conjunctive_query_genre (db, db->priv->iradio_genres, traversal_data);
 out:
 	g_free (traversal_data);
 }
@@ -1508,83 +1698,67 @@ split_query_by_disjunctions (RhythmDBTree *db, GPtrArray *query)
 	return conjunctions;
 }
 
+struct RhythmDBTreeQueryGatheringData
+{
+	RhythmDBTree *db;
+	GHashTable *entries;
+	RhythmDBQueryModel *main_model;
+};
+
 static void
-do_query_recurse (RhythmDBTree *db, GPtrArray *query, RhythmDBTreeTraversalFunc func, gpointer data)
+do_query_recurse (RhythmDBTree *db, GPtrArray *query, RhythmDBTreeTraversalFunc func,
+		  struct RhythmDBTreeQueryGatheringData *data, gboolean *cancel)
 {
 	GList *conjunctions, *tem;
 
 	conjunctions = split_query_by_disjunctions (db, query);
 
+	rb_debug ("doing recursive query, %d conjunctions", g_list_length (conjunctions));
+
+	/* If there is a disjunction involved, we must uniquify the entry hits. */
+	if (conjunctions->next != NULL)
+		data->entries = g_hash_table_new (g_direct_hash, g_direct_equal);
+	else
+		data->entries = NULL;
+
 	for (tem = conjunctions; tem; tem = tem->next) {
-		conjunctive_query (db, tem->data, func, data);
+		if (G_UNLIKELY (*cancel))
+			break;
+		conjunctive_query (db, tem->data, func, data, cancel);
 		g_ptr_array_free (tem->data, TRUE);
 	}
 
+	if (data->entries != NULL)
+		g_hash_table_destroy (data->entries);
+
 	g_list_free (conjunctions);
 }
 
-struct RhythmDBTreeQueryGatheringData
-{
-	RhythmDBTree *db;
-	GHashTable *genres;
-	GHashTable *artists;
-	GHashTable *albums;
-
-	GtkListStore *main_model;
-};
-
 static void
 handle_entry_match (RhythmDB *db, RhythmDBTreeEntry *entry,
 		    struct RhythmDBTreeQueryGatheringData *data)
 {
-	GtkTreeIter iter;
-	gtk_list_store_prepend (data->main_model, &iter);
-	gtk_list_store_set (data->main_model, &iter, 0, RHYTHMDB_TREE_ENTRY (entry), -1);
 
-	g_hash_table_insert (data->genres, get_entry_genre_name (entry), NULL);
-	g_hash_table_insert (data->artists, get_entry_artist_name (entry), NULL);
-	g_hash_table_insert (data->albums, get_entry_album_name (entry), NULL);
-}
-
-static void
-fill_list_store_from_hash (char *key, gpointer unused, GtkListStore *store)
-{
-	GtkTreeIter iter;
-	gtk_list_store_prepend (store, &iter);
-	gtk_list_store_set (store, &iter, 0, key, -1);
+	if (data->entries
+	    && g_hash_table_lookup (data->entries, entry))
+		return;
+		
+	rhythmdb_query_model_add_entry (data->main_model, entry);
 }
 
 static void
 rhythmdb_tree_do_full_query (RhythmDB *adb,
 			     GPtrArray *query,
-			     GtkTreeModel **main_model,
-			     GtkTreeModel **genre_model,
-			     GtkTreeModel **artist_model,
-			     GtkTreeModel **album_model)
+			     GtkTreeModel *main_model,
+			     gboolean *cancel)
 {
 	RhythmDBTree *db = RHYTHMDB_TREE (adb);
 	struct RhythmDBTreeQueryGatheringData *data = g_new (struct RhythmDBTreeQueryGatheringData, 1);
 
-	*main_model = GTK_TREE_MODEL (gtk_list_store_new (1, G_TYPE_POINTER));
-	data->main_model = GTK_LIST_STORE (*main_model);
-	data->genres = g_hash_table_new (g_direct_hash, g_direct_equal);
-	data->artists = g_hash_table_new (g_direct_hash, g_direct_equal);
-	data->albums = g_hash_table_new (g_direct_hash, g_direct_equal);
-	
-	do_query_recurse (db, query, (RhythmDBTreeTraversalFunc) handle_entry_match, data);
+	data->main_model = RHYTHMDB_QUERY_MODEL (main_model);
+
+	do_query_recurse (db, query, (RhythmDBTreeTraversalFunc) handle_entry_match, data, cancel);
 
-	*genre_model = GTK_TREE_MODEL (gtk_list_store_new (1, G_TYPE_STRING));
-	*artist_model = GTK_TREE_MODEL (gtk_list_store_new (1, G_TYPE_STRING));
-	*album_model = GTK_TREE_MODEL (gtk_list_store_new (1, G_TYPE_STRING));
-	g_hash_table_foreach (data->genres, (GHFunc) fill_list_store_from_hash,
-			      *genre_model);
-	g_hash_table_foreach (data->artists, (GHFunc) fill_list_store_from_hash,
-			      *artist_model);
-	g_hash_table_foreach (data->albums, (GHFunc) fill_list_store_from_hash,
-			      *album_model);
-	g_hash_table_destroy (data->genres);
-	g_hash_table_destroy (data->artists);
-	g_hash_table_destroy (data->albums);
 	g_free (data);
 }
 
@@ -1594,93 +1768,3 @@ rhythmdb_tree_entry_lookup_by_location (RhythmDB *adb, const char *uri)
 	RhythmDBTree *db = RHYTHMDB_TREE (adb);
 	return g_hash_table_lookup (db->priv->entries, uri);
 }
-
-
-/* static GtkListStore * */
-/* create_main_model (RhythmDB *db) */
-/* { */
-/* 	GtkListStore *store; */
-/* 	GType *types; */
-/* 	int i; */
-	
-/* 	source->priv->db = g_value_get_object (value); */
-	
-/* 	types = g_new (GType, RHYTHMDB_NUM_PROPERTIES); */
-	
-/* 	for (i = 0; i < RHYTHMDB_NUM_PROPERTIES; i++) */
-/* 		types[i] = rhythmdb_get_property_type (db, i); */
-
-/* 	store = g_object_new (GTK_TYPE_LIST_STORE, NULL); */
-	
-/* 	gtk_list_store_set_column_types (store, RHYTHMDB_NUM_PROPERTIES, types); */
-
-/* 	g_free (types); */
-/* 	return store; */
-/* } */
-/* static GtkTreeModel * */
-/* rhythmdb_tree_do_entry_query (RhythmDB *adb, GPtrArray *query) */
-/* { */
-/* 	RhythmDBTree *db = RHYTHMDB_TREE (adb); */
-/* 	GHashTable *result_set; */
-
-/* 	result_set = build_entry_query_set (db, query); */
-
-/* 	return GTK_TREE_MODEL (rhythmdb_entry_model_new_from_hash (result_set)); */
-/* } */
-
-/* struct RhythmDBTreePropertyGatheringData */
-/* { */
-/* 	guint prop_id; */
-/* 	GHashTable *set; */
-/* }; */
-
-/* static void */
-/* gather_property (RhythmDBTreeEntry *entry, gpointer unused, */
-/* 		 struct RhythmDBTreePropertyGatheringData *data) */
-/* { */
-/* 	g_hash_table_insert (data->set, RHYTHMDB_TREE_ENTRY_VALUE (entry, data->prop_id), NULL); */
-/* } */
-
-/* static GHashTable * */
-/* gather_property_set (RhythmDB *db, guint prop_id, GHashTable *table) */
-/* { */
-/* 	GHashTable *ret; */
-/* 	struct RhythmDBTreePropertyGatheringData *data; */
-
-/* 	ret = g_hash_table_new (g_direct_hash, g_direct_equal); */
-	
-/* 	data = g_new (struct RhythmDBTreePropertyGatheringData, 1); */
-/* 	data->prop_id = prop_id; */
-/* 	data->set = ret; */
-
-/* 	g_hash_table_foreach (table, gather_property, data); */
-
-/* 	g_free (data); */
-/* 	return ret; */
-/* } */
-
-/* static void */
-/* handle_entry_match (RhythmDB *db, RhythmDBTreeEntry *entry, GHashTable *set) */
-/* { */
-/* 	g_hash_table_insert (set, entry, NULL); */
-/* } */
-
-
-/* static GtkTreeModel * */
-/* rhythmdb_tree_do_property_query (RhythmDB *db, guint property_id, RhythmDBQuery *query) */
-/* { */
-/* 	GHashTable *result_set; */
-	
-/* 	switch (property_id) */
-/* 	{ */
-/* 	case RHYTHMDB_PROP_GENRE: */
-/* 	case RHYTHMDB_PROP_ARTIST: */
-/* 	case RHYTHMDB_PROP_ALBUM: */
-/* 		break; */
-		
-/* 	default: */
-/* 		g_assert_not_reached (); */
-/* 		break; */
-/* 	} */
-	
-/* } */
