@@ -27,7 +27,10 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <gtk/gtk.h>
+#include <bonobo/Bonobo.h>
 #include <bonobo/bonobo-main.h>
+#include <libbonobo.h>
+#include <bonobo/bonobo-property-bag-client.h>
 #include <glade/glade-init.h>
 #include <monkey-media.h>
 #include <stdlib.h>
@@ -152,6 +155,9 @@ main (int argc, char **argv)
 			
 			g_object_set_data (G_OBJECT (rb_shell), "argv", argv);
 			g_object_set_data (G_OBJECT (rb_shell), "argc", GINT_TO_POINTER (argc));
+
+			g_object_set_data (G_OBJECT (rb_shell), "rb-shell-no-registration",
+					   GINT_TO_POINTER (no_registration));
 			
 			g_idle_add ((GSourceFunc) rb_init, rb_shell);
 		}
@@ -205,13 +211,52 @@ rb_init (RBShell *shell)
 	argv = (char **) g_object_get_data (G_OBJECT (shell), "argv");
 	argc = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (shell), "argc"));
 
-	rb_handle_cmdline (argv, argc, FALSE);
+	if (!g_object_get_data (G_OBJECT (shell), "rb-shell-no-registration"))
+		rb_handle_cmdline (argv, argc, FALSE);
 	
 	GDK_THREADS_LEAVE ();
 
 	return FALSE;
 }
 
+static GNOME_Rhythmbox_SongInfo*
+get_song_info (GNOME_Rhythmbox shell)
+{
+	Bonobo_PropertyBag pb;
+	CORBA_any *any;
+	GNOME_Rhythmbox_SongInfo *song_info;
+
+	pb = GNOME_Rhythmbox_getPlayerProperties (shell, &ev);
+	if (BONOBO_EX (&ev)) {
+		char *err = bonobo_exception_get_text (&ev);
+		g_warning (_("An exception occured '%s'"), err);
+		CORBA_free (pb);
+		return NULL;
+	}
+
+	any = bonobo_pbclient_get_value (pb, "song", 
+					 TC_GNOME_Rhythmbox_SongInfo, 
+					 &ev);
+	if (BONOBO_EX (&ev)) {
+		char *err = bonobo_exception_get_text (&ev);
+		g_warning (_("An exception occured '%s'"), err);
+		g_free (err);
+		CORBA_free (pb);
+		return NULL;
+	}
+	
+	if ((any == NULL) || (!CORBA_TypeCode_equivalent (any->_type, TC_GNOME_Rhythmbox_SongInfo, NULL))) {
+		song_info = NULL;
+	} else {
+		song_info = (GNOME_Rhythmbox_SongInfo*)any->_value;
+		any->_release = FALSE;
+		CORBA_free (any);
+	}
+
+	CORBA_free (pb);
+	return song_info;
+}
+
 static void
 rb_handle_cmdline (char **argv, int argc,
 		   gboolean already_running)
@@ -219,6 +264,7 @@ rb_handle_cmdline (char **argv, int argc,
 	GNOME_Rhythmbox shell;
 	int i;
 	gboolean grab_focus = TRUE;
+	
 	shell = bonobo_activation_activate_from_id (RB_SHELL_OAFIID, 0, NULL, &ev);
 	if (shell == NULL)
 	{
@@ -231,20 +277,43 @@ rb_handle_cmdline (char **argv, int argc,
 
 	if (print_playing)
 	{
-		printf ("%s\n", GNOME_Rhythmbox_getPlayingTitle (shell, &ev));
+		GNOME_Rhythmbox_SongInfo *song_info;
+
+		song_info = get_song_info (shell);
+		if (song_info == NULL) {
+			g_print ("\n");
+		} else {
+			g_print ("%s\n", song_info->title);
+			CORBA_free (song_info);
+		}
 		grab_focus = FALSE;
+
 	}
 	
 	if (print_playing_path)
 	{
-		printf ("%s\n", GNOME_Rhythmbox_getPlayingPath (shell, &ev));
+		GNOME_Rhythmbox_SongInfo *song_info;
+
+		song_info = get_song_info (shell);
+		if (song_info == NULL) {
+			g_print ("\n");
+		} else {
+			g_print ("%s\n", song_info->path);
+			CORBA_free (song_info);
+		}
 		grab_focus = FALSE;
 	}
 	
 	if (print_song_length)
 	{
-		long song_length = GNOME_Rhythmbox_getPlayingSongDuration (shell, &ev);
-		printf ("%ld\n", song_length);
+		GNOME_Rhythmbox_SongInfo *song_info;
+		song_info = get_song_info (shell);
+		if (song_info == NULL) {
+			g_print ("-1\n");
+		} else {
+			g_print ("%d\n", song_info->duration);
+			CORBA_free (song_info);
+		}
 		grab_focus = FALSE;
 	}
 	
