@@ -2,6 +2,7 @@
  *  arch-tag: Header for object implementing main playback logic
  *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
+ *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,6 +21,7 @@
  */
 
 #include <gtk/gtkhbox.h>
+#include <bonobo/bonobo-arg.h>
 #include <bonobo/bonobo-ui-component.h>
 #include <monkey-media.h>
 
@@ -73,21 +75,22 @@ void			rb_shell_player_stop		(RBShellPlayer *player);
 void			rb_shell_player_do_previous	(RBShellPlayer *player);
 void			rb_shell_player_do_next		(RBShellPlayer *player);
 
-long			rb_shell_player_get_playing_time (RBShellPlayer *player);
-void			rb_shell_player_set_playing_time (RBShellPlayer *player, long time);
+long			rb_shell_player_get_playing_time(RBShellPlayer *player);
+void			rb_shell_player_set_playing_time(RBShellPlayer *player, long time);
 long			rb_shell_player_get_playing_song_duration (RBShellPlayer *player);
 
-void			rb_shell_player_set_shuffle	(RBShellPlayer *shell_player,
-							 gboolean shuffle);
-
 MonkeyMediaPlayer *	rb_shell_player_get_mm_player	(RBShellPlayer *shell_player);
 
 gboolean		rb_shell_player_get_playing	(RBShellPlayer *shell_player);
 
 const char *		rb_shell_player_get_playing_path(RBShellPlayer *shell_player);
 
-void			rb_shell_player_sync_buttons (RBShellPlayer *player);
+void			rb_shell_player_sync_buttons	(RBShellPlayer *player);
 
+void                    rb_shell_player_set_repeat      (RBShellPlayer *player,
+							 gboolean new_val);
+void                    rb_shell_player_set_shuffle     (RBShellPlayer *player,
+							 gboolean new_val);
 
 #ifdef HAVE_ACME
 gboolean		rb_shell_player_handle_key	(RBShellPlayer *player, guint keyval);
