@@ -20,6 +20,7 @@
  *
  */
 
+#include <bonobo/bonobo-arg.h>
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-context.h>
 #include <bonobo/bonobo-exception.h>
@@ -48,7 +49,7 @@
 #include "rb-shell.h"
 #include "rb-debug.h"
 #include "rb-dialog.h"
-#include "rb-node-db.h"
+#include "rhythmdb-tree.h"
 #include "rb-stock-icons.h"
 #include "rb-sourcelist.h"
 #include "rb-string-helpers.h"
@@ -65,9 +66,7 @@
 #include "rb-shell-preferences.h"
 #include "rb-playlist.h"
 #include "rb-bonobo-helpers.h"
-#include "rb-library.h"
 #include "rb-library-source.h"
-#include "rb-iradio-backend.h"
 #include "rb-load-failure-dialog.h"
 #include "rb-new-station-dialog.h"
 #include "rb-iradio-source.h"
@@ -77,7 +76,6 @@
 #include "rb-shell-preferences.h"
 #include "rb-playlist-source.h"
 #include "rb-file-monitor.h"
-#include "rb-library-dnd-types.h"
 #include "rb-thread-helpers.h"
 #include "eel-gconf-extensions.h"
 
@@ -94,10 +92,18 @@ static void rb_shell_corba_add_to_library (PortableServer_Servant _servant,
 					   CORBA_Environment *ev);
 static void rb_shell_corba_grab_focus (PortableServer_Servant _servant,
 				       CORBA_Environment *ev);
-static char * rb_shell_corba_get_playing_title (PortableServer_Servant _servant,
-						CORBA_Environment *ev);
-static char * rb_shell_corba_get_playing_path (PortableServer_Servant _servant,
+static void rb_shell_corba_playpause (PortableServer_Servant _servant,
 					       CORBA_Environment *ev);
+static void rb_shell_corba_next (PortableServer_Servant _servant,
+					       CORBA_Environment *ev);
+static void rb_shell_corba_previous (PortableServer_Servant _servant,
+					       CORBA_Environment *ev);
+static CORBA_long rb_shell_corba_get_playing_time (PortableServer_Servant _servant,
+						   CORBA_Environment *ev);
+static void rb_shell_corba_set_playing_time (PortableServer_Servant _servant,
+						   CORBA_long time, CORBA_Environment *ev);
+
+static Bonobo_PropertyBag rb_shell_corba_get_player_properties (PortableServer_Servant _servant, CORBA_Environment *ev);
 
 void rb_shell_handle_playlist_entry (RBShell *shell, GList *locations, const char *title,
 				     const char *genre);
@@ -120,10 +126,9 @@ static void source_selected_cb (RBSourceList *sourcelist,
 static void source_activated_cb (RBSourceList *sourcelist,
 				 RBSource *source,
 				 RBShell *shell);
-static void rb_shell_library_error_cb (RBLibrary *library,
-				       const char *uri, const char *msg,
-				       RBShell *shell); 
-static void rb_shell_library_progress_cb (RBLibrary *library, double progress, RBShell *shell); 
+static void rb_shell_db_error_cb (RhythmDB *db,
+				  const char *uri, const char *msg,
+				  RBShell *shell); 
 static void rb_shell_load_failure_dialog_response_cb (GtkDialog *dialog,
 						      int response_id,
 						      RBShell *shell);
@@ -132,20 +137,23 @@ static void rb_shell_playlist_added_cb (RBPlaylistManager *mgr, RBSource *source
 static void rb_shell_playlist_load_start_cb (RBPlaylistManager *mgr, RBShell *shell);
 static void rb_shell_playlist_load_finish_cb (RBPlaylistManager *mgr, RBShell *shell);
 static void rb_shell_source_deleted_cb (RBSource *source, RBShell *shell);
-static void rb_shell_set_window_title (RBShell *shell,
-			               const char *window_title);
+static void rb_shell_set_window_title (RBShell *shell, const char *window_title);
+static void rb_shell_set_duration (RBShell *shell, const char *duration);
 static void rb_shell_player_window_title_changed_cb (RBShellPlayer *player,
 					             const char *window_title,
 					             RBShell *shell);
+static void rb_shell_player_duration_changed_cb (RBShellPlayer *player,
+						 const char *duration,
+						 RBShell *shell);
 static void rb_shell_cmd_about (BonoboUIComponent *component,
 		                RBShell *shell,
 		                const char *verbname);
 static void rb_shell_cmd_contents (BonoboUIComponent *component,
 		                RBShell *shell,
 		                const char *verbname);
-static void rb_shell_cmd_close (BonoboUIComponent *component,
-		                RBShell *shell,
-			        const char *verbname);
+static void rb_shell_cmd_quit (BonoboUIComponent *component,
+			       RBShell *shell,
+			       const char *verbname);
 static void rb_shell_cmd_preferences (BonoboUIComponent *component,
 		                      RBShell *shell,
 		                      const char *verbname);
@@ -176,6 +184,8 @@ static void rb_shell_view_smalldisplay_changed_cb (BonoboUIComponent *component,
 						 Bonobo_UIComponent_EventType type,
 						 const char *state,
 						 RBShell *shell);
+static void rb_shell_load_complete_cb (RhythmDB *db, RBShell *shell);
+static void rb_shell_legacy_load_complete_cb (RhythmDB *db, RBShell *shell);
 static void rb_shell_sync_sourcelist_visibility (RBShell *shell);
 static void rb_shell_sync_smalldisplay (RBShell *shell);
 static void sourcelist_visibility_changed_cb (GConfClient *client,
@@ -204,11 +214,7 @@ static gboolean rb_shell_show_popup_cb (RBSourceList *sourcelist,
 					RBShell *shell);
 static void tray_deleted_cb (GtkWidget *win, GdkEventAny *event, RBShell *shell);
 
-static const GtkTargetEntry target_table[] =
-	{
-		{ RB_LIBRARY_DND_URI_LIST_TYPE, 0, RB_LIBRARY_DND_URI_LIST },
-		{ RB_LIBRARY_DND_NODE_ID_TYPE,  0, RB_LIBRARY_DND_NODE_ID }
-	};
+static const GtkTargetEntry target_table[] = { { "text/uri-list", 0,0 } };
 
 #define CMD_PATH_VIEW_SMALLDISPLAY "/commands/ToggleSmallDisplay"
 #define CMD_PATH_VIEW_SOURCELIST   "/commands/ShowSourceList"
@@ -236,19 +242,19 @@ struct RBShellPrivate
 
 	GList *sources;
 
+	RhythmDB *db;
+
 	RBShellPlayer *player_shell;
 	RBShellClipboard *clipboard_shell;
 	RBSourceHeader *source_header;
 	RBStatusbar *statusbar;
 	RBPlaylistManager *playlist_manager;
 
-	RBLibrary *library;
 	RBSource *library_source;
 	GtkWidget *load_error_dialog;
 	GList *supported_media_extensions;
-	gboolean show_library_errors;
+	gboolean show_db_errors;
 
-	RBIRadioBackend *iradio_backend;
  	RBIRadioSource *iradio_source;
 
 #ifdef HAVE_AUDIOCD
@@ -260,13 +266,20 @@ struct RBShellPrivate
 	GtkWidget *prefs;
 
 	RBTrayIcon *tray_icon;
+
+	BonoboPropertyBag *pb;
+
+	char *cached_title;
+	char *cached_duration;
+
+	gulong song_changed_cb_id;
 };
 
 static BonoboUIVerb rb_shell_verbs[] =
 {
 	BONOBO_UI_VERB ("About",        (BonoboUIVerbFn) rb_shell_cmd_about),
 	BONOBO_UI_VERB ("Contents",	(BonoboUIVerbFn) rb_shell_cmd_contents),
-	BONOBO_UI_VERB ("Close",        (BonoboUIVerbFn) rb_shell_cmd_close),
+	BONOBO_UI_VERB ("Quit",		(BonoboUIVerbFn) rb_shell_cmd_quit),
 	BONOBO_UI_VERB ("Preferences",  (BonoboUIVerbFn) rb_shell_cmd_preferences),
 	BONOBO_UI_VERB ("AddToLibrary", (BonoboUIVerbFn) rb_shell_cmd_add_to_library),
 	BONOBO_UI_VERB ("AddLocation",  (BonoboUIVerbFn) rb_shell_cmd_add_location),
@@ -330,8 +343,12 @@ rb_shell_class_init (RBShellClass *klass)
 	epv->handleFile   = rb_shell_corba_handle_file;
 	epv->addToLibrary = rb_shell_corba_add_to_library;
 	epv->grabFocus    = rb_shell_corba_grab_focus;
-	epv->getPlayingTitle = rb_shell_corba_get_playing_title;
-	epv->getPlayingPath = rb_shell_corba_get_playing_path;
+	epv->playPause = rb_shell_corba_playpause;
+	epv->previous = rb_shell_corba_previous;
+	epv->next = rb_shell_corba_next;
+	epv->getPlayingTime = rb_shell_corba_get_playing_time;
+	epv->setPlayingTime = rb_shell_corba_set_playing_time;
+	epv->getPlayerProperties = rb_shell_corba_get_player_properties;
 }
 
 static void
@@ -385,13 +402,16 @@ rb_shell_finalize (GObject *object)
 
 	gtk_widget_destroy (shell->priv->window);
 
-	rb_library_shutdown (shell->priv->library);
+	rb_debug ("shutting down DB");
+	rhythmdb_shutdown (shell->priv->db);
 
-	rb_debug ("unreffing library");
-	g_object_unref (G_OBJECT (shell->priv->library));
+	rb_debug ("saving db");
+	rhythmdb_read_lock (shell->priv->db);
+	rhythmdb_save (shell->priv->db);
+	rhythmdb_read_unlock (shell->priv->db);
 
-	rb_debug ("unreffing iradio backend");
-	g_object_unref (G_OBJECT (shell->priv->iradio_backend));
+	rb_debug ("unreffing DB");
+	g_object_unref (G_OBJECT (shell->priv->db));
 
 	if (shell->priv->prefs != NULL)
 		gtk_widget_destroy (shell->priv->prefs);
@@ -436,10 +456,11 @@ handle_playlist_entry_cb (RBPlaylist *playlist, const char *uri, const char *tit
 	/* We assume all HTTP is iradio.  This is probably a broken assumption,
 	 * but it's very difficult to really fix...
 	 */
-	if (rb_uri_is_iradio (uri) != FALSE)
-		rb_iradio_backend_add_station_full (shell->priv->iradio_backend, uri, title, genre);
-	else
-		rb_library_add_uri (shell->priv->library, (char *) uri);
+	if (rb_uri_is_iradio (uri) != FALSE) {
+		rb_iradio_source_add_station (shell->priv->iradio_source, uri, title, genre);
+	} else {
+		rhythmdb_add_uri_async (shell->priv->db, (char *) uri);
+	}
 }
 
 static void
@@ -464,7 +485,7 @@ rb_shell_corba_handle_file (PortableServer_Servant _servant,
 	 * the library.
 	 */
 	if (!rb_playlist_parse (parser, uri))
-		rb_library_add_uri (shell->priv->library, uri);
+		rhythmdb_add_uri_async (shell->priv->db, uri);
 	g_object_unref (G_OBJECT (parser));
 }
 
@@ -475,7 +496,7 @@ rb_shell_corba_add_to_library (PortableServer_Servant _servant,
 {
 	RBShell *shell = RB_SHELL (bonobo_object (_servant));
 
-	rb_library_add_uri (shell->priv->library, uri);
+	rhythmdb_add_uri_async (shell->priv->db, uri);
 }
 
 static void
@@ -488,52 +509,232 @@ rb_shell_corba_grab_focus (PortableServer_Servant _servant,
 	gtk_widget_grab_focus (shell->priv->window);
 }
 
-static CORBA_char *
-rb_shell_corba_get_playing_title (PortableServer_Servant _servant,
-				  CORBA_Environment *ev)
-{
-	RBShell *shell = RB_SHELL (bonobo_object (_servant));
-	const char *str;
-	CORBA_char *ret;
+enum PlayerProperties {
+	PROP_VISIBILITY,
+	PROP_SONG
+};
 
-	GDK_THREADS_ENTER ();
-	str = gtk_window_get_title (GTK_WINDOW (shell->priv->window));
-	ret = CORBA_string_alloc (strlen (str));
-	strcpy (ret, str);
-	GDK_THREADS_LEAVE ();
-	return ret;
+
+static void 
+shell_notify_pb_changes (RBShell *shell, const gchar *property_name, 
+			 BonoboArg *arg) 
+{
+	if (shell->priv->pb != NULL) {
+		bonobo_event_source_notify_listeners_full (shell->priv->pb->es,
+							   "Bonobo/Property",
+							   "change",
+							   property_name,
+							   arg, NULL);
+	}
 }
 
-static CORBA_char *
-rb_shell_corba_get_playing_path (PortableServer_Servant _servant,
+static GNOME_Rhythmbox_SongInfo *
+get_song_info_from_player (RBShell *shell)
+{
+	RhythmDBEntry *entry;
+	RhythmDB *db = shell->priv->db;
+	GNOME_Rhythmbox_SongInfo *song_info;
+	RBEntryView *view;
+
+	view = rb_source_get_entry_view (shell->priv->selected_source);
+	g_object_get (G_OBJECT (view), "playing_entry", &entry, NULL);
+	if (entry == NULL) {
+		/* FIXME: ORBit probably won't like it */
+		return NULL;
+	}
+
+	song_info = GNOME_Rhythmbox_SongInfo__alloc ();
+	rhythmdb_read_lock (db);
+	song_info->title = CORBA_string_dup (rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_TITLE));
+	song_info->artist = CORBA_string_dup (rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_ARTIST));
+	song_info->genre = CORBA_string_dup (rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_GENRE));
+	song_info->album = CORBA_string_dup (rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_ALBUM));
+	song_info->path = CORBA_string_dup (rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_LOCATION));
+	song_info->track_number = rhythmdb_entry_get_int (db, entry, RHYTHMDB_PROP_TRACK_NUMBER);
+	song_info->duration = rhythmdb_entry_get_long (db, entry, RHYTHMDB_PROP_DURATION);
+	song_info->bitrate = rhythmdb_entry_get_int (db, entry, RHYTHMDB_PROP_QUALITY);
+	song_info->filesize = rhythmdb_entry_get_long (db, entry, RHYTHMDB_PROP_FILE_SIZE);
+	song_info->rating = rhythmdb_entry_get_int (db, entry, RHYTHMDB_PROP_RATING);
+	song_info->play_count = rhythmdb_entry_get_int (db, entry, RHYTHMDB_PROP_PLAY_COUNT);
+	song_info->last_played = rhythmdb_entry_get_long (db, entry, RHYTHMDB_PROP_LAST_PLAYED);
+	rhythmdb_read_unlock (db);
+
+	return song_info;
+}
+
+static void
+shell_pb_get_prop (BonoboPropertyBag *bag,
+		   BonoboArg         *arg,
+		   guint              arg_id,
+		   CORBA_Environment *ev,
+		   gpointer           user_data)
+{
+	RBShell *shell = RB_SHELL (user_data);
+	RBShellPlayer *player;
+
+	player = RB_SHELL_PLAYER (shell->priv->player_shell);
+
+	switch (arg_id) {
+
+	case PROP_VISIBILITY:
+		BONOBO_ARG_SET_BOOLEAN (arg, FALSE);
+		break;
+
+	case PROP_SONG: {
+		GNOME_Rhythmbox_SongInfo *ret_val;
+		ret_val = get_song_info_from_player (shell);
+		(GNOME_Rhythmbox_SongInfo*)arg->_value = ret_val;
+		if (ret_val == NULL) {
+			arg->_type = TC_null;
+		} else {
+			arg->_type = TC_GNOME_Rhythmbox_SongInfo;
+		}
+		break;		
+	}
+
+	default:
+		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
+		break;
+	}
+}
+
+static void
+shell_pb_set_prop (BonoboPropertyBag *bag,
+		   const BonoboArg   *arg,
+		   guint              arg_id,
+		   CORBA_Environment *ev,
+		   gpointer           user_data)
+{
+	switch (arg_id) {
+
+	case PROP_VISIBILITY:
+		break;
+
+	case PROP_SONG:
+		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_ReadOnly);
+		break;
+
+	default:
+		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
+		break;
+	}
+}
+
+
+static Bonobo_PropertyBag
+rb_shell_corba_get_player_properties (PortableServer_Servant _servant, 
+				      CORBA_Environment *ev)
+{	
+	RBShell *shell = RB_SHELL (bonobo_object (_servant));
+
+	if (shell->priv->pb == NULL) {
+		gchar *params_to_map[] = {"repeat", "shuffle", "playing", 
+					  NULL};
+		GParamSpec **params;
+		int length = 0;
+		int i = 0;
+
+		shell->priv->pb = bonobo_property_bag_new (shell_pb_get_prop, 
+							   shell_pb_set_prop, 
+							   shell);
+		
+		
+		/* Map RBShellPlayer properties to bonobo properties */
+		while (params_to_map[length] != NULL) {
+			length++;
+		}
+		params = malloc (length * sizeof (GParamSpec *));
+		for (i = 0; i < length; i++) {
+			params[i] = g_object_class_find_property (G_OBJECT_CLASS (RB_SHELL_PLAYER_GET_CLASS (shell->priv->player_shell)), params_to_map[i]);
+		}
+		bonobo_property_bag_map_params (shell->priv->pb,
+						G_OBJECT (shell->priv->player_shell),
+						(const GParamSpec **)params, length);
+
+
+		/* Manually install the other properties */
+		bonobo_property_bag_add (shell->priv->pb, "visibility", 
+					 PROP_VISIBILITY, BONOBO_ARG_BOOLEAN, NULL, 
+					 _("Whether the main window is visible"), 0);
+
+		bonobo_property_bag_add (shell->priv->pb, "song", 
+					 PROP_SONG, TC_GNOME_Rhythmbox_SongInfo, NULL, 
+					 _("Properties for the current song"), 0);
+	}
+	/* If the creation of the property bag failed, 
+	 * return a corba exception
+	 */
+	
+	return CORBA_Object_duplicate (BONOBO_OBJREF (shell->priv->pb), NULL);
+}
+
+static void
+rb_shell_corba_playpause (PortableServer_Servant _servant,
+			  CORBA_Environment *ev)
+{
+	RBShell *shell = RB_SHELL (bonobo_object (_servant));
+	rb_shell_player_playpause (shell->priv->player_shell);
+}
+
+static void
+rb_shell_corba_next (PortableServer_Servant _servant,
+		     CORBA_Environment *ev)
+{
+	RBShell *shell = RB_SHELL (bonobo_object (_servant));
+	rb_shell_player_do_next (shell->priv->player_shell);
+}
+
+static void
+rb_shell_corba_previous (PortableServer_Servant _servant,
+			 CORBA_Environment *ev)
+{
+	RBShell *shell = RB_SHELL (bonobo_object (_servant));
+	rb_shell_player_do_previous (shell->priv->player_shell);
+}
+
+static CORBA_long
+rb_shell_corba_get_playing_time (PortableServer_Servant _servant,
 				 CORBA_Environment *ev)
 {
 	RBShell *shell = RB_SHELL (bonobo_object (_servant));
-	const char *str;
-	CORBA_char *ret;
-	
-	GDK_THREADS_ENTER ();
-	str = rb_shell_player_get_playing_path (shell->priv->player_shell);
-	if (str == NULL)
-		str = "";
-	ret = CORBA_string_alloc (strlen (str));
-	strcpy (ret, str);
-	GDK_THREADS_LEAVE ();
-	return ret;
+	return rb_shell_player_get_playing_time (shell->priv->player_shell);
 }
 
-static gboolean
-async_library_release_brakes (RBShell *shell)
+static void
+rb_shell_corba_set_playing_time (PortableServer_Servant _servant,
+				 CORBA_long time, CORBA_Environment *ev)
 {
-	rb_debug ("async releasing library brakes");
+	RBShell *shell = RB_SHELL (bonobo_object (_servant));
+	rb_shell_player_set_playing_time (shell->priv->player_shell, time);
+}
 
-	GDK_THREADS_ENTER ();
 
-	rb_library_release_brakes (shell->priv->library);
+static void
+rb_shell_property_changed_generic_cb (GObject *object,
+				      GParamSpec *pspec, 
+				      RBShell *shell)
+{
+	BonoboArg *arg = bonobo_arg_new (TC_CORBA_boolean);
+	gboolean value;
 
-	GDK_THREADS_LEAVE ();
+	g_object_get (object, pspec->name, &value, NULL);
+	BONOBO_ARG_SET_BOOLEAN (arg, value);
+	shell_notify_pb_changes (shell, pspec->name, arg);
+	/* FIXME: arg should be released somehow */
+}
 
-	return FALSE;
+static void
+rb_shell_entry_changed_cb (GObject *object, GParamSpec *pspec, RBShell *shell)
+{
+	GNOME_Rhythmbox_SongInfo *song_info;
+	BonoboArg *arg;
+	
+	g_assert (strcmp (pspec->name, "playing-entry") == 0);
+	song_info = get_song_info_from_player (shell);
+	arg = bonobo_arg_new (TC_GNOME_Rhythmbox_SongInfo);
+	(GNOME_Rhythmbox_SongInfo*)arg->_value = song_info;
+	shell_notify_pb_changes (shell, "song", arg);
+	/* FIXME: arg should be released somehow */
 }
 
 void
@@ -545,6 +746,7 @@ rb_shell_construct (RBShell *shell)
 	BonoboWindow *win;
 	Bonobo_UIContainer corba_container;
 	GtkWidget *vbox;
+	gboolean rhythmdb_exists;
 	
 	g_return_if_fail (RB_IS_SHELL (shell));
 
@@ -565,7 +767,7 @@ rb_shell_construct (RBShell *shell)
 	g_signal_connect (G_OBJECT (win), "delete_event",
 			  G_CALLBACK (rb_shell_window_delete_cb),
 			  shell);
-
+  
 	rb_debug ("shell: creating container area");
 	shell->priv->container = bonobo_window_get_ui_container (win);
 
@@ -588,8 +790,31 @@ rb_shell_construct (RBShell *shell)
 			       "rhythmbox-ui.xml",
 			       "rhythmbox", NULL);
 
-	rb_debug ("shell: creating library");
-	shell->priv->library = rb_library_new ();
+	/* Initialize the database */
+	rb_debug ("creating database object");
+	{
+		char *fname = g_build_filename (rb_dot_dir (), "rhythmdb.xml", NULL);
+
+		rhythmdb_exists = g_file_test (fname, G_FILE_TEST_EXISTS);
+		
+		shell->priv->db = rhythmdb_tree_new (fname);
+		g_free (fname);
+
+		if (rhythmdb_exists) {
+			g_signal_connect_object (G_OBJECT (shell->priv->db), "load-complete",
+						 G_CALLBACK (rb_shell_load_complete_cb), shell,
+						 0);
+			rhythmdb_load (shell->priv->db);
+		}
+	}
+
+	if (!rhythmdb_exists && eel_gconf_get_boolean (CONF_FIRST_TIME)) {
+		rb_debug ("loading legacy library db");
+		g_signal_connect_object (G_OBJECT (shell->priv->db), "legacy-load-complete",
+					 G_CALLBACK (rb_shell_legacy_load_complete_cb), shell,
+					 0);
+		rhythmdb_load_legacy (shell->priv->db);
+	}
 
 	rb_debug ("shell: setting up tray icon");
 	tray_deleted_cb (NULL, NULL, shell);
@@ -612,11 +837,28 @@ rb_shell_construct (RBShell *shell)
 		shell->priv->player_shell = rb_shell_player_new (shell->priv->ui_component,
 								 tray_component);
 	}
+	g_signal_connect (G_OBJECT (shell->priv->player_shell), 
+			  "notify::repeat", 
+			  G_CALLBACK (rb_shell_property_changed_generic_cb), 
+			  shell);
+	g_signal_connect (G_OBJECT (shell->priv->player_shell), 
+			  "notify::shuffle", 
+			  G_CALLBACK (rb_shell_property_changed_generic_cb), 
+			  shell);
+	g_signal_connect (G_OBJECT (shell->priv->player_shell), 
+			  "notify::playing", 
+			  G_CALLBACK (rb_shell_property_changed_generic_cb), 
+			  shell);
 	g_signal_connect (G_OBJECT (shell->priv->player_shell),
 			  "window_title_changed",
 			  G_CALLBACK (rb_shell_player_window_title_changed_cb),
 			  shell);
-	shell->priv->clipboard_shell = rb_shell_clipboard_new (shell->priv->ui_component);
+	g_signal_connect (G_OBJECT (shell->priv->player_shell),
+			  "duration_changed",
+			  G_CALLBACK (rb_shell_player_duration_changed_cb),
+			  shell);
+	shell->priv->clipboard_shell = rb_shell_clipboard_new (shell->priv->ui_component,
+							       shell->priv->db);
 	shell->priv->source_header = rb_source_header_new (shell->priv->ui_component);
 
 	shell->priv->paned = gtk_hpaned_new ();
@@ -629,7 +871,9 @@ rb_shell_construct (RBShell *shell)
 	g_signal_connect (G_OBJECT (shell->priv->sourcelist), "show_popup",
 			  G_CALLBACK (rb_shell_show_popup_cb), shell);
 
-	shell->priv->statusbar = rb_statusbar_new (shell->priv->ui_component);
+	shell->priv->statusbar = rb_statusbar_new (shell->priv->db,
+						   shell->priv->ui_component,
+						   shell->priv->player_shell);
 
 	rb_sourcelist_set_dnd_targets (RB_SOURCELIST (shell->priv->sourcelist), target_table,
 				       G_N_ELEMENTS (target_table));
@@ -684,35 +928,33 @@ rb_shell_construct (RBShell *shell)
 	rb_shell_sync_sourcelist_visibility (shell);
 
 	shell->priv->load_error_dialog = rb_load_failure_dialog_new ();
-	shell->priv->show_library_errors = FALSE;
+	shell->priv->show_db_errors = FALSE;
 	gtk_widget_hide (shell->priv->load_error_dialog);
 
 	shell->priv->supported_media_extensions = monkey_media_get_supported_filename_extensions ();
 
-	g_signal_connect (G_OBJECT (shell->priv->library), "error",
-			  G_CALLBACK (rb_shell_library_error_cb), shell);
-	g_signal_connect (G_OBJECT (shell->priv->library), "progress",
-			  G_CALLBACK (rb_shell_library_progress_cb), shell);
+	g_signal_connect (G_OBJECT (shell->priv->db), "error",
+			  G_CALLBACK (rb_shell_db_error_cb), shell);
 
 	g_signal_connect (G_OBJECT (shell->priv->load_error_dialog), "response",
 			  G_CALLBACK (rb_shell_load_failure_dialog_response_cb), shell);
 
 	/* initialize sources */
 	rb_debug ("shell: creating library source");
-	shell->priv->library_source = rb_library_source_new (shell->priv->library);
+	shell->priv->library_source = rb_library_source_new (shell->priv->db);
 	rb_shell_append_source (shell, shell->priv->library_source);
 
-	rb_debug ("shell: creating iradio backend");
-	shell->priv->iradio_backend = g_object_new (RB_TYPE_IRADIO_BACKEND, NULL);
 	rb_debug ("shell: creating iradio source");
-	shell->priv->iradio_source = RB_IRADIO_SOURCE (rb_iradio_source_new (shell->priv->iradio_backend));
+	shell->priv->iradio_source = RB_IRADIO_SOURCE (rb_iradio_source_new (shell->priv->db));
+	
 	rb_shell_append_source (shell, RB_SOURCE (shell->priv->iradio_source));
 
 	shell->priv->playlist_manager = rb_playlist_manager_new (shell->priv->ui_component,
 								 GTK_WINDOW (shell->priv->window),
-								 shell->priv->library,
+								 shell->priv->db,
+								 RB_SOURCELIST (shell->priv->sourcelist),
 								 RB_LIBRARY_SOURCE (shell->priv->library_source),
-								 shell->priv->iradio_backend);
+								 RB_IRADIO_SOURCE (shell->priv->iradio_source));
 
 	g_signal_connect (G_OBJECT (shell->priv->playlist_manager), "playlist_added",
 			  G_CALLBACK (rb_shell_playlist_added_cb), shell);
@@ -727,13 +969,6 @@ rb_shell_construct (RBShell *shell)
 
 	rb_shell_select_source (shell, shell->priv->library_source); /* select this one by default */
 
-	/* load library */
-	rb_debug ("shell: loading library");
-	rb_library_load (shell->priv->library);
-
-	rb_debug ("shell: loading iradio");
- 	rb_iradio_backend_load (shell->priv->iradio_backend);
-
 #ifdef HAVE_AUDIOCD
         if (rb_audiocd_is_any_device_available () == TRUE) {
 		rb_debug ("AudioCD device is available");
@@ -755,33 +990,32 @@ rb_shell_construct (RBShell *shell)
 		rb_debug ("No AudioCD device is available!");
 #endif
 	
-	/* register with CORBA */
-	CORBA_exception_init (&ev);
-	
-	corba_object = bonobo_object_corba_objref (BONOBO_OBJECT (shell));
-	if (bonobo_activation_active_server_register (RB_SHELL_OAFIID, corba_object) != Bonobo_ACTIVATION_REG_SUCCESS)
-		registration_failed = TRUE;
-
-	if (bonobo_activation_active_server_register (RB_FACTORY_OAFIID, corba_object) != Bonobo_ACTIVATION_REG_SUCCESS)
-		registration_failed = TRUE;
-
-	if (registration_failed) {
-		/* this is not critical, but worth a warning nevertheless */
-		char *msg = rb_shell_corba_exception_to_string (&ev);
-		g_message (_("Failed to register the shell: %s\n"
-			     "This probably means that you installed Rhythmbox in a\n"
-			     "different prefix than bonobo-activation; this\n"
-			     "warning is harmless, but IPC will not work.\n"), msg);
-		g_free (msg);
-	}
+	if (!g_object_get_data (G_OBJECT (shell), "rb-shell-no-registration")) {
+		/* register with CORBA */
+		CORBA_exception_init (&ev);
 		
-	CORBA_exception_free (&ev);
+		corba_object = bonobo_object_corba_objref (BONOBO_OBJECT (shell));
 
-	rb_debug ("Registered with Bonobo Activation");
-
-	/* now that the lib is loaded, we can load the music playlists */
-	rb_debug ("shell: loading playlists");
-	rb_playlist_manager_load_playlists (shell->priv->playlist_manager);
+		if (bonobo_activation_active_server_register (RB_SHELL_OAFIID, corba_object) != Bonobo_ACTIVATION_REG_SUCCESS)
+			registration_failed = TRUE;
+		
+		if (bonobo_activation_active_server_register (RB_FACTORY_OAFIID, corba_object) != Bonobo_ACTIVATION_REG_SUCCESS)
+			registration_failed = TRUE;
+		
+		if (registration_failed) {
+			/* this is not critical, but worth a warning nevertheless */
+			char *msg = rb_shell_corba_exception_to_string (&ev);
+			g_message (_("Failed to register the shell: %s\n"
+				     "This probably means that you installed Rhythmbox in a "
+				     "different prefix than bonobo-activation; this "
+				     "warning is harmless, but IPC will not work."), msg);
+			g_free (msg);
+		
+		}
+		CORBA_exception_free (&ev);
+		
+		rb_debug ("Registered with Bonobo Activation");
+	}
 
 	/* GO GO GO! */
 	rb_debug ("shell: syncing window state");
@@ -789,8 +1023,8 @@ rb_shell_construct (RBShell *shell)
 	gtk_widget_show_all (GTK_WIDGET (shell->priv->tray_icon));
 
 	/* Stop here if this is the first time. */
-	if (!eel_gconf_get_boolean(CONF_FIRST_TIME)) {
-		RBDruid *druid = rb_druid_new (shell->priv->library);
+	if (!eel_gconf_get_boolean (CONF_FIRST_TIME)) {
+		RBDruid *druid = rb_druid_new (shell->priv->db);
 		gtk_widget_hide (GTK_WIDGET (shell->priv->window));
 		rb_druid_show (druid);
 		g_object_unref (G_OBJECT (druid));
@@ -799,7 +1033,6 @@ rb_shell_construct (RBShell *shell)
 	rb_statusbar_sync_state (shell->priv->statusbar);
 	rb_shell_sync_smalldisplay (shell);
 	gtk_widget_show (GTK_WIDGET (shell->priv->window));
-	g_idle_add ((GSourceFunc) async_library_release_brakes, shell);
 }
 
 char *
@@ -933,26 +1166,19 @@ source_activated_cb (RBSourceList *sourcelist,
 }
 
 static void
-rb_shell_library_progress_cb (RBLibrary *library, double progress, RBShell *shell)
-{
-	rb_debug ("setting progress to %f", progress);
-	rb_statusbar_set_progress (shell->priv->statusbar, progress);
-}
-
-static void
-rb_shell_library_error_cb (RBLibrary *library,
-			   const char *uri, const char *msg,
-			   RBShell *shell)
+rb_shell_db_error_cb (RhythmDB *db,
+		      const char *uri, const char *msg,
+		      RBShell *shell)
 {
 	GList *tem;
 	GnomeVFSURI *vfsuri;
 	gchar *basename;
 	gssize baselen;
 
-	rb_debug ("got library error, showing: %s",
-		  shell->priv->show_library_errors ? "TRUE" : "FALSE");
+	rb_debug ("got db error, showing: %s",
+		  shell->priv->show_db_errors ? "TRUE" : "FALSE");
 	
-	if (!shell->priv->show_library_errors)
+	if (!shell->priv->show_db_errors)
 		return;
 
 	vfsuri = gnome_vfs_uri_new (uri);
@@ -983,7 +1209,7 @@ rb_shell_load_failure_dialog_response_cb (GtkDialog *dialog,
 					  RBShell *shell)
 {
 	rb_debug ("got response");
-	shell->priv->show_library_errors = FALSE;
+	shell->priv->show_db_errors = FALSE;
 }
 
 static void
@@ -1014,13 +1240,13 @@ rb_shell_playlist_added_cb (RBPlaylistManager *mgr, RBSource *source, RBShell *s
 static void
 rb_shell_playlist_load_start_cb (RBPlaylistManager *mgr, RBShell *shell)
 {
-	shell->priv->show_library_errors = TRUE;
+	shell->priv->show_db_errors = TRUE;
 }
 
 static void
 rb_shell_playlist_load_finish_cb (RBPlaylistManager *mgr, RBShell *shell)
 {
-	shell->priv->show_library_errors = FALSE;
+	shell->priv->show_db_errors = FALSE;
 }
 
 static void
@@ -1047,12 +1273,24 @@ static void
 rb_shell_select_source (RBShell *shell,
 			RBSource *source)
 {
+	RBEntryView *view;
+
 	if (shell->priv->selected_source == source)
 		return;
 
 	rb_debug ("selecting source %p", source);
-
+	
+	if (shell->priv->song_changed_cb_id != 0) {
+		view = rb_source_get_entry_view (shell->priv->selected_source);
+		g_signal_handler_disconnect (view, 
+					     shell->priv->song_changed_cb_id);
+	}
 	shell->priv->selected_source = source;
+	
+	view = rb_source_get_entry_view (shell->priv->selected_source);
+	shell->priv->song_changed_cb_id = 
+		g_signal_connect (view, "notify::playing-entry", 
+				  (GCallback)rb_shell_entry_changed_cb, shell);
 
 	/* show source */
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (shell->priv->notebook),
@@ -1083,44 +1321,74 @@ rb_shell_player_window_title_changed_cb (RBShellPlayer *player,
 }
 
 static void
-rb_shell_set_window_title (RBShell *shell,
-			   const char *window_title)
+rb_shell_player_duration_changed_cb (RBShellPlayer *player,
+				     const char *duration,
+				     RBShell *shell)
 {
+	rb_shell_set_duration (shell, duration);
+}
+
+static void
+rb_shell_set_duration (RBShell *shell, const char *duration)
+{
+	gboolean playing = rb_shell_player_get_playing (shell->priv->player_shell);
+	char *tooltip;
+		
+	if (shell->priv->cached_title == NULL) 
+		tooltip = g_strdup (_("Not playing"));
+	else if (!playing) {
+		tooltip = g_strdup_printf (_("%s\nPaused, %s remaining"),
+					 shell->priv->cached_title, duration);
+	} else {
+		tooltip = g_strdup_printf (_("%s\n%s remaining"),
+					   shell->priv->cached_title, duration);
+	}
+	
+	rb_tray_icon_set_tooltip (shell->priv->tray_icon, tooltip);
+	g_free (tooltip);
+}
+
+static void
+rb_shell_set_window_title (RBShell *shell, const char *window_title)
+{
+	g_free (shell->priv->cached_title);
+	
 	if (window_title == NULL) {
 		rb_debug ("clearing title");
+
+		shell->priv->cached_title = NULL;
+		
 		gtk_window_set_title (GTK_WINDOW (shell->priv->window),
 				      _("Music Player"));
-
-		rb_tray_icon_set_tooltip (shell->priv->tray_icon, _("Not playing"));
 	}
 	else {
 		gboolean playing = rb_shell_player_get_playing (shell->priv->player_shell);
 		char *title;
 
-		if (!strcmp (gtk_window_get_title (GTK_WINDOW (shell->priv->window)),
-			     window_title))
-		    return;
+		if (shell->priv->cached_title &&
+		    !strcmp (shell->priv->cached_title, window_title)) {
+			shell->priv->cached_title = g_strdup (window_title);
+			return;
+		}
+		shell->priv->cached_title = g_strdup (window_title);
 
 		rb_debug ("setting title to \"%s\"", window_title);
 		if (!playing) {
-			char *tmp;
 			title = g_strdup_printf (_("%s (Paused)"), window_title);
 			gtk_window_set_title (GTK_WINDOW (shell->priv->window),
 					      title);
-
-			tmp = g_strdup_printf (_("%s\nPaused"), window_title);
-			rb_tray_icon_set_tooltip (shell->priv->tray_icon, tmp);
-			g_free (tmp);
+			g_free (title);
 		} else {
-			title = g_strdup (window_title);
 			gtk_window_set_title (GTK_WINDOW (shell->priv->window),
 					      window_title);
+
+/* This simply doesn't do anything at the moment, because the notification area
+ * is broken.
+ */
 #if 0			
 			egg_tray_icon_send_message (EGG_TRAY_ICON (shell->priv->tray_icon),
 						    3000, window_title, -1);
 #endif
-
-			rb_tray_icon_set_tooltip (shell->priv->tray_icon, window_title);
 		}
 	}
 }
@@ -1162,6 +1430,9 @@ rb_shell_cmd_about (BonoboUIComponent *component,
 #include "MAINTAINERS.tab"
 		"",
 		NULL,
+#include "MAINTAINERS.old.tab"
+		"",
+		NULL,
 #include "AUTHORS.tab"
 		NULL
 	};
@@ -1181,14 +1452,12 @@ rb_shell_cmd_about (BonoboUIComponent *component,
 	pixbuf = gdk_pixbuf_new_from_file (rb_file ("about-logo.png"), NULL);
 
 	authors[0] = _("Maintainers:");
-	tem = authors;
-	while (1) {
-		if (*tem == NULL) {
-			*tem = _("Contributors:");
-			break;
-		}
-		tem++;
-	}
+	for (tem = authors; *tem != NULL; tem++)
+		;
+	*tem = _("Former Maintainers:");
+	for (; *tem != NULL; tem++)
+		;
+	*tem = _("Contributors:");
 
 	{
 		const char *backend;
@@ -1229,9 +1498,9 @@ rb_shell_cmd_about (BonoboUIComponent *component,
 }
 
 static void
-rb_shell_cmd_close (BonoboUIComponent *component,
-		    RBShell *shell,
-		    const char *verbname)
+rb_shell_cmd_quit (BonoboUIComponent *component,
+		   RBShell *shell,
+		   const char *verbname)
 {
 	rb_shell_quit (shell);
 }
@@ -1298,12 +1567,12 @@ ask_file_response_cb (GtkDialog *dialog,
 		g_free (stored);
 	}
 
-	shell->priv->show_library_errors = TRUE;
+	shell->priv->show_db_errors = TRUE;
     
 	while (*filecur != NULL) {
 		if (g_utf8_validate (*filecur, -1, NULL)) {
 			char *uri = gnome_vfs_get_uri_from_local_path (*filecur);
-			rb_library_add_uri (shell->priv->library, uri);
+			rhythmdb_add_uri_async (shell->priv->db, uri);
 			g_free (uri);
 		}
 		filecur++;
@@ -1337,7 +1606,7 @@ rb_shell_cmd_add_location (BonoboUIComponent *component,
 			   RBShell *shell,
 			   const char *verbname)
 {
-	shell->priv->show_library_errors = TRUE;
+	shell->priv->show_db_errors = TRUE;
 
 	rb_library_source_add_location (RB_LIBRARY_SOURCE (shell->priv->library_source),
 					GTK_WINDOW (shell->priv->window));
@@ -1350,7 +1619,7 @@ rb_shell_cmd_new_station (BonoboUIComponent *component,
 {
 	GtkWidget *dialog;
 	rb_debug ("Got new station command");
-	dialog = rb_new_station_dialog_new (shell->priv->iradio_backend);
+	dialog = rb_new_station_dialog_new (shell->priv->db);
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 }
@@ -1383,6 +1652,22 @@ rb_shell_quit (RBShell *shell)
 	bonobo_object_unref (BONOBO_OBJECT (shell));
 }
 
+static void
+rb_shell_load_complete_cb (RhythmDB *db, RBShell *shell)
+{
+	rb_debug ("load complete");
+	GDK_THREADS_ENTER ();
+	rb_playlist_manager_load_playlists (shell->priv->playlist_manager);
+	GDK_THREADS_LEAVE ();
+}
+
+static void
+rb_shell_legacy_load_complete_cb (RhythmDB *db, RBShell *shell)
+{
+	rb_debug ("legacy load complete");
+	rb_playlist_manager_load_legacy_playlists (shell->priv->playlist_manager);
+}
+
 static void
 rb_shell_sync_sourcelist_visibility (RBShell *shell)
 {
@@ -1465,7 +1750,8 @@ rb_shell_sync_paned (RBShell *shell)
 		if (maximized) {
 			gtk_window_get_size (GTK_WINDOW (shell->priv->window), &actual_width, NULL);
 			default_width =  eel_gconf_get_integer (CONF_STATE_WINDOW_WIDTH);
-			pos = pos * (float)actual_width/(float)default_width + 1;            
+			if (actual_width != default_width)
+				pos = pos * (float)actual_width/(float)default_width + 1;            
 		}
 		gtk_paned_set_position (GTK_PANED (shell->priv->paned),
 					pos);
@@ -1505,13 +1791,6 @@ paned_changed_cb (GConfClient *client,
 }
 
 
-/* static void */
-/* handle_songs_func (RBNode *node, */
-/* 		   RBPlaylistSource *playlist) */
-/* { */
-/* 	rb_playlist_source_add_node (playlist, node); */
-/* } */
-
 static void
 sourcelist_drag_received_cb (RBSourceList *sourcelist,
 			     RBSource *source,
@@ -1571,22 +1850,23 @@ static void
 rb_shell_jump_to_current (RBShell *shell)
 {
 	RBSource *source = rb_shell_player_get_source (shell->priv->player_shell);
-	RBNodeView *songs;
-	RBNode *playing;
+	RBEntryView *songs;
+	RhythmDBEntry *playing;
 
 	g_return_if_fail (source != NULL);
 
-	songs = rb_source_get_node_view (source);
+	songs = rb_source_get_entry_view (source);
 
-	playing = rb_node_view_get_playing_node (songs);
+	playing = rb_entry_view_get_playing_entry (songs);
 
 	g_return_if_fail (playing != NULL);
 
-	if (!rb_node_view_get_node_visible (songs, playing)) {
+	if (!rb_entry_view_get_entry_visible (songs, playing)) {
+		rb_source_reset_filters (source);
 		rb_source_header_clear_search (shell->priv->source_header);
 	}
 	rb_shell_select_source (shell, source);
-	if (!rb_node_view_get_node_visible (songs, playing)) {
+	if (!rb_entry_view_get_entry_visible (songs, playing)) {
 		rb_source_search (shell->priv->selected_source, NULL);
 	}
 	rb_shell_player_jump_to_current (shell->priv->player_shell);
@@ -1612,7 +1892,7 @@ tray_deleted_cb (GtkWidget *win, GdkEventAny *event, RBShell *shell)
 	rb_debug ("creating new tray icon");
 	shell->priv->tray_icon = rb_tray_icon_new (shell->priv->container,
 						   shell->priv->ui_component,
-						   shell->priv->library,
+						   shell->priv->db,
 						   GTK_WINDOW (shell->priv->window));
 	g_signal_connect (G_OBJECT (shell->priv->tray_icon), "delete_event",
 			  G_CALLBACK (tray_deleted_cb), shell);
