@@ -24,10 +24,12 @@
 #define __RB_PLAYLIST_SOURCE_H
 
 #include <bonobo/bonobo-ui-container.h>
+#include <libxml/tree.h>
 
 #include "rb-source.h"
-#include "rb-library.h"
+#include "rhythmdb.h"
 #include "rb-library-source.h"
+#include "rhythmdb-query-model.h"
 
 G_BEGIN_DECLS
 
@@ -54,25 +56,33 @@ typedef struct
 
 GType		rb_playlist_source_get_type	(void);
 
-RBSource *	rb_playlist_source_new		(RBLibrary *library,
-						 RBLibrarySource *libsource);
+RBSource *	rb_playlist_source_new		(RhythmDB *db, gboolean smart);
 
-RBSource *	rb_playlist_source_new_from_file(RBLibrary *library,
-						 RBLibrarySource *libsource,
-						 const char *file);
+RBSource *	rb_playlist_source_new_from_xml	(RhythmDB *db,
+						 xmlNodePtr node);
 
-void		rb_playlist_source_do_rename	(RBPlaylistSource *source, GtkWindow *parent);
+RhythmDBQueryModel *rb_playlist_source_get_model(RBPlaylistSource *source);
 
-const char *	rb_playlist_source_get_file	(RBPlaylistSource *group);
-
-void		rb_playlist_source_save		(RBPlaylistSource *source);
-void		rb_playlist_source_load		(RBPlaylistSource *source);
+void		rb_playlist_source_set_query	(RBPlaylistSource *source, GPtrArray *query,
+						 guint limit);
 
 void		rb_playlist_source_save_playlist(RBPlaylistSource *source,
 						 const char *uri);
 
-void		rb_playlist_source_add_node	(RBPlaylistSource *source, 
-						 RBNode *node);
+void		rb_playlist_source_save_to_xml	(RBPlaylistSource *source,
+						 xmlNodePtr node);
+
+void		rb_playlist_source_add_entry	(RBPlaylistSource *source,
+						 RhythmDBEntry *entry);
+
+void		rb_playlist_source_remove_entry	(RBPlaylistSource *source,
+						 RhythmDBEntry *entry);
+
+void		rb_playlist_source_add_location	(RBPlaylistSource *source,
+						 const char *location);
+
+void		rb_playlist_source_remove_location	(RBPlaylistSource *source,
+						 	 const char *location);
 
 void		rb_playlist_source_delete	(RBPlaylistSource *source);
 
