@@ -24,10 +24,12 @@
 #include <bonobo/bonobo-ui-component.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <libgnome/gnome-i18n.h>
+#include <time.h>
 
 #include "rb-source.h"
 #include "rb-debug.h"
 #include "rb-dialog.h"
+#include "rb-cut-and-paste-code.h"
 #include "rb-bonobo-helpers.h"
 
 static void rb_source_class_init (RBSourceClass *klass);
@@ -44,9 +46,11 @@ static void rb_source_get_property (GObject *object,
 
 static const char * default_get_browser_key (RBSource *status);
 static GList *default_get_extra_views (RBSource *source);
+static gboolean default_can_rename (RBSource *source);
 static gboolean default_can_search (RBSource *source);
 static gboolean default_can_cut (RBSource *source);
 static GList *default_copy (RBSource *source);
+static void default_reset_filters (RBSource *source);
 static void default_song_properties (RBSource *source);
 static GtkWidget * default_get_config_widget (RBSource *source);
 static RBSourceEOFType default_handle_eos (RBSource *source);
@@ -122,11 +126,13 @@ rb_source_class_init (RBSourceClass *klass)
 
 	klass->impl_get_browser_key = default_get_browser_key;
 	klass->impl_get_extra_views = default_get_extra_views;
+	klass->impl_can_rename = default_can_rename;
 	klass->impl_can_search = default_can_search;
 	klass->impl_can_cut = default_can_cut;
 	klass->impl_can_delete = default_can_cut;
 	klass->impl_can_copy = default_can_cut;
 	klass->impl_copy = default_copy;
+	klass->impl_reset_filters = default_reset_filters;
 	klass->impl_song_properties = default_song_properties;
 	klass->impl_handle_eos = default_handle_eos;
 	klass->impl_buffering_done = default_buffering_done;
@@ -239,7 +245,6 @@ rb_source_get_property (GObject *object,
 	}
 }
 
-
 const char *
 rb_source_get_status (RBSource *status)
 {
@@ -274,12 +279,49 @@ rb_source_notify_filter_changed (RBSource *status)
 	g_signal_emit (G_OBJECT (status), rb_source_signals[FILTER_CHANGED], 0);
 }
 
-RBNodeView *
-rb_source_get_node_view (RBSource *source)
+void
+rb_source_update_play_statistics (RBSource *source, RhythmDB *db, RhythmDBEntry *entry)
+{
+	char *time_string;
+	time_t now;
+	GValue value = { 0, };
+
+	g_value_init (&value, G_TYPE_INT);
+
+	rhythmdb_write_lock (db);
+	
+	g_value_set_int (&value, rhythmdb_entry_get_int (db, entry,
+							 RHYTHMDB_PROP_PLAY_COUNT) + 1);
+
+	/* Increment current play count */
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_PLAY_COUNT, &value);
+	g_value_unset (&value);
+	
+	/* Reset the last played time */
+	time (&now);
+
+	g_value_init (&value, G_TYPE_LONG);
+	g_value_set_long (&value, now);
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_LAST_PLAYED, &value);
+	g_value_unset (&value);
+
+	time_string = eel_strdup_strftime (_("%Y-%m-%d %H:%M"), localtime (&now));
+
+	g_value_init (&value, G_TYPE_STRING);
+	g_value_set_string (&value, time_string);
+	g_free (time_string);
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_LAST_PLAYED_STR, &value);
+	g_value_unset (&value);
+
+	rhythmdb_write_unlock (db);
+}
+
+RBEntryView *
+rb_source_get_entry_view (RBSource *source)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 
-	return klass->impl_get_node_view (source);
+	return klass->impl_get_entry_view (source);
 }
 
 static GList *
@@ -304,6 +346,19 @@ rb_source_get_pixbuf (RBSource *source)
 	return klass->impl_get_pixbuf (source);
 }
 
+static gboolean
+default_can_rename (RBSource *source)
+{
+	return FALSE;
+}
+
+gboolean
+rb_source_can_rename (RBSource *source)
+{
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+
+	return klass->impl_can_rename (source);
+}
 
 static gboolean
 default_can_search (RBSource *source)
@@ -382,7 +437,7 @@ rb_source_cut (RBSource *source)
 static GList *
 default_copy (RBSource *source)
 {
-	return g_list_copy (rb_node_view_get_selection (rb_source_get_node_view (source)));
+	return g_list_copy (rb_entry_view_get_selected_entries (rb_source_get_entry_view (source)));
 }
 	
 GList *
@@ -409,6 +464,20 @@ rb_source_delete (RBSource *source)
 	klass->impl_delete (source);
 }
 
+static void
+default_reset_filters (RBSource *source)
+{
+	rb_debug ("no implementation of reset_filters for this source");
+}
+
+void
+rb_source_reset_filters (RBSource *source)
+{
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+
+	klass->impl_reset_filters (source);
+}
+
 static void
 default_song_properties (RBSource *source)
 {
