@@ -26,7 +26,7 @@
 #include <gtk/gtkhbox.h>
 #include <bonobo/bonobo-ui-component.h>
 
-#include "rb-node-view.h"
+#include "rb-entry-view.h"
 
 G_BEGIN_DECLS
 
@@ -66,14 +66,16 @@ typedef struct
 
 	const char *	(*impl_get_browser_key)	(RBSource *source);
 
-	RBNodeView *	(*impl_get_node_view)	(RBSource *source);
+	RBEntryView *	(*impl_get_entry_view)	(RBSource *source);
 	GList *		(*impl_get_extra_views)	(RBSource *source);
 
 	GdkPixbuf *	(*impl_get_pixbuf)	(RBSource *source);
+	gboolean	(*impl_can_rename)	(RBSource *source);
 
 	gboolean	(*impl_can_search)	(RBSource *source);
 
 	void		(*impl_search)		(RBSource *source, const char *text);
+	void		(*impl_reset_filters)	(RBSource *source);
 	GtkWidget *	(*impl_get_config_widget)(RBSource *source);
 
 	gboolean	(*impl_can_cut)		(RBSource *source);
@@ -82,7 +84,7 @@ typedef struct
 	
 	GList *		(*impl_cut)		(RBSource *source);
 	GList *		(*impl_copy)		(RBSource *source);
-	void		(*impl_paste)		(RBSource *source, GList *nodes);
+	void		(*impl_paste)		(RBSource *source, GList *entries);
 	void		(*impl_delete)		(RBSource *source);
 
 	void		(*impl_song_properties)	(RBSource *source);
@@ -110,22 +112,28 @@ void		rb_source_notify_filter_changed	(RBSource *source);
 
 void		rb_source_notify_status_changed (RBSource *status);
 
+void		rb_source_update_play_statistics(RBSource *source, RhythmDB *db,
+						 RhythmDBEntry *entry);
+
 /* general interface */
 const char *	rb_source_get_status		(RBSource *source);
 
 const char *	rb_source_get_browser_key	(RBSource *source);
 
-RBNodeView *	rb_source_get_node_view		(RBSource *source);
+RBEntryView *	rb_source_get_entry_view	(RBSource *source);
 
 GList *		rb_source_get_extra_views	(RBSource *source);
 
 GdkPixbuf *	rb_source_get_pixbuf		(RBSource *source);
+gboolean	rb_source_can_rename		(RBSource *source);
 
 gboolean	rb_source_can_search		(RBSource *source);
 
 void		rb_source_search		(RBSource *source,
 						 const char *text);
 
+void		rb_source_reset_filters		(RBSource *source);
+
 GtkWidget *	rb_source_get_config_widget	(RBSource *source);
 
 gboolean	rb_source_can_cut		(RBSource *source);
@@ -134,7 +142,7 @@ gboolean	rb_source_can_copy		(RBSource *source);
 
 GList *		rb_source_cut			(RBSource *source);
 GList *		rb_source_copy			(RBSource *source);
-void		rb_source_paste			(RBSource *source, GList *nodes);
+void		rb_source_paste			(RBSource *source, GList *entries);
 void		rb_source_delete		(RBSource *source);
 
 void		rb_source_song_properties	(RBSource *source);
